/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editcountertype.hpp"
#include "ff_grouptree.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/buffer.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditCounterType) FF_EditCounterTypeMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditCounterType::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditCounterType::onCmdCancel),

};


FXIMPLEMENT(FF_EditCounterType, FXDialogBox, FF_EditCounterTypeMap, ARRAYNUMBER(FF_EditCounterTypeMap))



FF_EditCounterType::FF_EditCounterType(FF_App* a,
                                       FXComposite *parent,
                                       const FXString& title,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h,
                                       FXint pl, FXint pr, FXint pt, FXint pb,
                                       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_nameEdit(NULL)
,m_descriptionEdit(NULL)
,m_xLabelEdit(NULL)
,m_yLabelEdit(NULL)
,m_zLabelEdit(NULL)
,m_xUnitsEdit(NULL)
,m_yUnitsEdit(NULL)
,m_zUnitsEdit(NULL)
,m_groupTree(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXIcon *ic;

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("AE_CounterType|Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 40, this, ID_EDIT_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterType|Description"), NULL, LABEL_NORMAL);
  m_descriptionEdit=new FXTextField(m, 40, this, ID_EDIT_DESCRIPTION,
                                    TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterType|X-Label"), NULL, LABEL_NORMAL);
  m_xLabelEdit=new FXTextField(m, 40, this, ID_EDIT_XLABEL,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterType|Y-Label"), NULL, LABEL_NORMAL);
  m_yLabelEdit=new FXTextField(m, 40, this, ID_EDIT_YLABEL,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterType|Z-Label"), NULL, LABEL_NORMAL);
  m_zLabelEdit=new FXTextField(m, 40, this, ID_EDIT_ZLABEL,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterType|X-Units"), NULL, LABEL_NORMAL);
  m_xUnitsEdit=new FXTextField(m, 40, this, ID_EDIT_XUNITS,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterType|Y-Units"), NULL, LABEL_NORMAL);
  m_yUnitsEdit=new FXTextField(m, 40, this, ID_EDIT_YUNITS,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterType|Z-Units"), NULL, LABEL_NORMAL);
  m_zUnitsEdit=new FXTextField(m, 40, this, ID_EDIT_ZUNITS,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);


  gb=new FXGroupBox(vf, I18N("CounterType Group"),
                    LAYOUT_FILL_X | LAYOUT_FILL_Y |
                    FRAME_LINE);
  m_groupTree=new FF_GroupTree(gb, this, ID_GROUP,
                               LAYOUT_FILL_X | LAYOUT_FILL_Y |
                               FOLDINGLIST_SINGLESELECT |
                               FOLDINGLIST_SHOWS_LINES |
                               FOLDINGLIST_SHOWS_BOXES |
                               FOLDINGLIST_ROOT_BOXES);
  m_groupTree->appendHeader(I18N("Group Name"));
  m_groupTree->setHeaderSize(0, m_nameEdit->getDefaultWidth());

  ic=a->getIcon("minifolderopen");
  if (ic)
    m_groupTree->setIconOpen(ic);
  ic=a->getIcon("minifolder");
  if (ic)
    m_groupTree->setIconClosed(ic);

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditCounterType::FF_EditCounterType()
:FXDialogBox()
,m_app(NULL)
,m_nameEdit(NULL)
,m_descriptionEdit(NULL)
,m_xLabelEdit(NULL)
,m_yLabelEdit(NULL)
,m_zLabelEdit(NULL)
,m_xUnitsEdit(NULL)
,m_yUnitsEdit(NULL)
,m_zUnitsEdit(NULL)
,m_groupTree(NULL)
{

}




FF_EditCounterType::~FF_EditCounterType() {
}



FXint FF_EditCounterType::getDefaultHeight() {
  /* add minimum height of table to be usefull */
  return FXDialogBox::getDefaultHeight()+100;
}



void FF_EditCounterType::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditCounterType");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  GWEN_DB_Group_free(db);
}



void FF_EditCounterType::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditCounterType", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



int FF_EditCounterType::checkGui(FXbool quiet) {
  FXString s;

  s=m_nameEdit->getText().simplify();
  if (s.empty()) {
    DBG_ERROR(0, "Empty name");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
                          I18N("Missing counter type name"));
      m_nameEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }

  return 0;
}



int FF_EditCounterType::toGui(const AE_COUNTER_TYPE *ct) {
  const char *s;
  AE_BOOK *b;

#define TXTTOGUI(n, m)              \
  s=AE_CounterType_Get##m(ct);         \
  if (s)                            \
    n->setText(s);

  TXTTOGUI(m_nameEdit, Name);
  TXTTOGUI(m_descriptionEdit, Description);
  TXTTOGUI(m_xLabelEdit, XLabel);
  TXTTOGUI(m_yLabelEdit, YLabel);
  TXTTOGUI(m_zLabelEdit, ZLabel);
#undef TXTTOGUI

  s=AE_Units_toString(AE_CounterType_GetXUnits(ct));
  if (s)
    m_xUnitsEdit->setText(s);

  s=AE_Units_toString(AE_CounterType_GetYUnits(ct));
  if (s)
    m_yUnitsEdit->setText(s);

  s=AE_Units_toString(AE_CounterType_GetZUnits(ct));
  if (s)
    m_zUnitsEdit->setText(s);


  b=m_app->getBook();
  if (b) {
    AE_GROUP_TREE *gt;
    int rv;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_CounterTypeGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      DBG_ERROR(0, "Adding %d groups", AE_Group_Tree_GetCount(gt));
      m_groupTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);
    m_groupTree->sortItems();
    m_groupTree->expandItems();
  }

  if (AE_CounterType_GetGroupId(ct)) {
    FF_GroupTreeItem *item=NULL;

    item=m_groupTree->findGroupItem(AE_CounterType_GetGroupId(ct));
    if (item)
      item->setSelected(true);
  }

  return 0;
}



int FF_EditCounterType::fromGui(AE_COUNTER_TYPE *ct) {
  FXString str;
  AE_GROUP *gr;

#define TXTFROMGUI(n, m)                   \
  str=n->getText().simplify();             \
  if (str.empty())                         \
    AE_CounterType_Set##m(ct, NULL);         \
  else                                     \
    AE_CounterType_Set##m(ct, str.text())

  TXTFROMGUI(m_nameEdit, Name);
  TXTFROMGUI(m_descriptionEdit, Description);
  TXTFROMGUI(m_xLabelEdit, XLabel);
  TXTFROMGUI(m_yLabelEdit, YLabel);
  TXTFROMGUI(m_zLabelEdit, YLabel);
#undef TXTFROMGUI

  str=m_xUnitsEdit->getText().simplify();
  if (str.empty())
    AE_CounterType_SetXUnits(ct, AE_Units_None);
  else
    AE_CounterType_SetXUnits(ct, AE_Units_fromString(str.text()));

  str=m_yUnitsEdit->getText().simplify();
  if (str.empty())
    AE_CounterType_SetYUnits(ct, AE_Units_None);
  else
    AE_CounterType_SetYUnits(ct, AE_Units_fromString(str.text()));

  str=m_zUnitsEdit->getText().simplify();
  if (str.empty())
    AE_CounterType_SetZUnits(ct, AE_Units_None);
  else
    AE_CounterType_SetZUnits(ct, AE_Units_fromString(str.text()));


  gr=m_groupTree->getFirstSelectedGroup();
  if (gr) {
    DBG_ERROR(0, "Current group: %d", AE_Group_GetId(gr));
    AE_CounterType_SetGroupId(ct, AE_Group_GetId(gr));
  }
  else {
    DBG_ERROR(0, "No current group");
    AE_CounterType_SetGroupId(ct, 0);
  }

  return 0;
}



void FF_EditCounterType::create() {
  FXDialogBox::create();
}



long FF_EditCounterType::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditCounterType::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditCounterType::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditCounterType::editCounterType(FF_App* app,
                                         FXComposite *parent,
                                         const FXString &title,
                                         AE_COUNTER_TYPE *ct) {
  FF_EditCounterType dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  res=dlg.toGui(ct);
  if (res)
    return false;
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
  dlg.init();
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(ct);
  if (res==0)
    return true;

  return false;
}









