/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editcountervalue.hpp"
#include "ff_dateedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/buffer.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditCounterValue) FF_EditCounterValueMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditCounterValue::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditCounterValue::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditCounterValue::ID_BUTTON_ANOTHER, FF_EditCounterValue::onCmdWantAnother),

};


FXIMPLEMENT(FF_EditCounterValue, FXDialogBox, FF_EditCounterValueMap, ARRAYNUMBER(FF_EditCounterValueMap))



FF_EditCounterValue::FF_EditCounterValue(FF_App* a,
                                         FXComposite *parent,
                                         const FXString& title,
                                         FXuint opts,
                                         FXint x, FXint y, FXint w, FXint h,
                                         FXint pl, FXint pr, FXint pt, FXint pb,
                                         FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_dateEdit(NULL)
,m_xValueEdit(NULL)
,m_yValueEdit(NULL)
,m_zValueEdit(NULL)
,m_noteEdit(NULL)
,m_wantAnother(false)
,m_anotherButton(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);


  new FXLabel(m, I18N("AE_CounterValue|Date"), NULL, LABEL_NORMAL);
  m_dateEdit=new FF_DateEdit(m, this, ID_EDIT_DATE,
                             I18N("YYYY/MM/DD"),
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterValue|X-Value"), NULL, LABEL_NORMAL);
  m_xValueEdit=new FXTextField(m, 40, this, ID_EDIT_XVALUE,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterValue|Y-Value"), NULL, LABEL_NORMAL);
  m_yValueEdit=new FXTextField(m, 40, this, ID_EDIT_YVALUE,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterValue|Z-Value"), NULL, LABEL_NORMAL);
  m_zValueEdit=new FXTextField(m, 40, this, ID_EDIT_YVALUE,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("AE_CounterValue|Note"), NULL, LABEL_NORMAL);
  m_noteEdit=new FXTextField(m, 40, this, ID_EDIT_NOTE,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new FXSpring(hf, LAYOUT_FILL_X);
  if (opts & OPTIONS_MULTI)
    m_anotherButton=new THEMEBUTTON(hf, I18N("FF_EditCounterValue|Another"), NULL, this, ID_BUTTON_ANOTHER, BUTTON_NORMAL);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT, BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL, BUTTON_NORMAL);
}



FF_EditCounterValue::FF_EditCounterValue()
:FXDialogBox()
,m_app(NULL)
,m_dateEdit(NULL)
,m_xValueEdit(NULL)
,m_yValueEdit(NULL)
,m_zValueEdit(NULL)
,m_noteEdit(NULL)
,m_wantAnother(false)
,m_anotherButton(NULL)
{
}




FF_EditCounterValue::~FF_EditCounterValue() {
}



FXint FF_EditCounterValue::getDefaultHeight() {
  /* add minimum height of table to be usefull */
  return FXDialogBox::getDefaultHeight()+100;
}



void FF_EditCounterValue::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditCounterValue");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  GWEN_DB_Group_free(db);
}



void FF_EditCounterValue::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditCounterValue", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



bool FF_EditCounterValue::_checkValueAndUnits(FXbool quiet, FXbool needed, FXTextField *tf, const FXString &errorTitle) {
  FXString str;

  str=tf->getText().simplify();
  if (!str.empty()) {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    errorTitle.text(),
			    "%s",
			    I18N("Two types of komma detected, "
				 "please specify the decimal point only"));
	tf->setFocus();
      }
      return false;
    }
    else {
      AQDB_VALUE *v=NULL;
      int units=0;
      int i;
      int rv;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(str.text(), &v, &units);
      if (rv<0) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              errorTitle.text(),
                              "%s",
                              I18N("Invalid value in input"));
          tf->setFocus();
        }
        return false;
      }
      AQDB_Value_free(v);
    }
  }
  else if (needed) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  errorTitle.text(),
			  "%s",
			  I18N("Missing input."));
      tf->setFocus();
    }
    return false;
  }

  return true;
}



int FF_EditCounterValue::_strToValue(FXbool needed, const FXString &str, AQDB_VALUE **pValue, int *pUnits) {
  if (str.empty()) {
    if (needed) {
      DBG_ERROR(0, "No data when needed");
      return GWEN_ERROR_NO_DATA;
    }
    return 0;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      int i;
      FXString nstr;
      int rv;

      /* replace textual comma with decimal point */
      nstr=str;
      i=nstr.find(',');
      if (i>=0)
	nstr[i]='.';

      rv=AE_Utils_ValueAndUnitsFromString(nstr.text(), pValue, pUnits);
      return rv;
    }
  }

  return 0;
}



int FF_EditCounterValue::checkGui(FXbool quiet) {
  GWEN_DATE *ti;
  FXString s;

  if (!_checkValueAndUnits(quiet, TRUE, m_xValueEdit, I18N("FF_EditCounterValue|Bad value for X")))
    return GWEN_ERROR_BAD_DATA;

  if (!_checkValueAndUnits(quiet, FALSE, m_yValueEdit, I18N("FF_EditCounterValue|Bad value for Y")))
    return GWEN_ERROR_BAD_DATA;

  if (!_checkValueAndUnits(quiet, FALSE, m_zValueEdit, I18N("FF_EditCounterValue|Bad value for Z")))
    return GWEN_ERROR_BAD_DATA;

  if (!m_dateEdit->getDate(&ti)) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Bad date input"));
      m_dateEdit->setFocus();
    }
    return GWEN_ERROR_BAD_DATA;
  }
  else
    GWEN_Date_free(ti);

  return 0;
}



int FF_EditCounterValue::toGui(const AE_COUNTER_VALUE *cv) {
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    const AQDB_VALUE *v;
    const GWEN_DATE *dt;

    v=AE_CounterValue_GetValueX(cv);
    if (v) {
      GWEN_BUFFER *tbuf;
  
      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AE_Utils_ValueAndUnitsToString(v, 0, tbuf, 2);
      m_xValueEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }

    v=AE_CounterValue_GetValueY(cv);
    if (v) {
      GWEN_BUFFER *tbuf;
  
      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AE_Utils_ValueAndUnitsToString(v, 0, tbuf, 2);
      m_yValueEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }

    v=AE_CounterValue_GetValueZ(cv);
    if (v) {
      GWEN_BUFFER *tbuf;
  
      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AE_Utils_ValueAndUnitsToString(v, 0, tbuf, 2);
      m_zValueEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_free(tbuf);
    }

  
    dt=AE_CounterValue_GetDate(cv);
    if (dt)
      m_dateEdit->setDate(dt);

    return 0;
  }
  else {
    DBG_ERROR(0, "Internal error: No book??");
    return GWEN_ERROR_INTERNAL;
  }

  return 0;
}



int FF_EditCounterValue::fromGui(AE_COUNTER_VALUE *cv) {
  FXString str;
  int rv;
  AQDB_VALUE *v;
  int units;

  /* x value */
  str=m_xValueEdit->getText().simplify();
  rv=_strToValue(TRUE, str, &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "Bad amount");
    return rv;
  }
  AE_CounterValue_SetValueX(cv, v);
  AQDB_Value_free(v);

  /* y value */
  str=m_yValueEdit->getText().simplify();
  rv=_strToValue(TRUE, str, &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "Bad amount");
    return rv;
  }
  AE_CounterValue_SetValueY(cv, v);
  AQDB_Value_free(v);

  /* z value */
  str=m_zValueEdit->getText().simplify();
  rv=_strToValue(TRUE, str, &v, &units);
  if (rv<0) {
    DBG_ERROR(0, "Bad amount");
    return rv;
  }
  AE_CounterValue_SetValueZ(cv, v);
  AQDB_Value_free(v);


  if (m_dateEdit) {
    GWEN_DATE *ti;

    if (!m_dateEdit->getDate(&ti)) {
      DBG_ERROR(0, "Bad date");
      return GWEN_ERROR_BAD_DATA;
    }
    AE_CounterValue_SetDate(cv, ti);
    GWEN_Date_free(ti);
  }

  return 0;
}



void FF_EditCounterValue::create() {
  FXDialogBox::create();
}



long FF_EditCounterValue::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditCounterValue::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    m_wantAnother=false;
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditCounterValue::onCmdWantAnother(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    m_wantAnother=true;
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditCounterValue::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditCounterValue::editCounterValue(FF_App* app,
                                           FXComposite *parent,
                                           const FXString &title,
                                           AE_COUNTER_VALUE *cv) {
  FF_EditCounterValue dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  res=dlg.toGui(cv);
  if (res)
    return false;
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
  dlg.init();
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(cv);
  if (res==0)
    return true;

  return false;
}



bool FF_EditCounterValue::createCounterValues(FF_App* app,
                                              FXComposite *parent,
                                              const FXString &title,
                                              const AE_COUNTER_VALUE *origTmpl) {
  FF_EditCounterValue dlg(app, parent, title, DECOR_ALL|FF_EditCounterValue::OPTIONS_MULTI);
  FXuint dlgres;
  FXint res;
  AE_COUNTER_VALUE *tmpl;
  AE_BOOK *b;

  b=app->getBook();
  if (b) {
    dlg.create();
    dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());
    dlg.init();
    dlg.show(PLACEMENT_VISIBLE);
  
    tmpl=AE_CounterValue_dup(origTmpl);
    for(;;) {
      AE_COUNTER_VALUE *cv;
      int rv;
  
      cv=AE_CounterValue_dup(tmpl);
      res=dlg.toGui(cv);
      if (res) {
	dlg.fini();
	AE_CounterValue_free(cv);
	AE_CounterValue_free(tmpl);
	dlg.hide();
	return false;
      }
  
      dlg.m_wantAnother=false;
      dlg.m_dateEdit->setFocus();
      dlgres=dlg.execute(PLACEMENT_VISIBLE);
      DBG_ERROR(0, "Result: %d", dlgres);
      if (dlgres==0) {
	dlg.fini();
	AE_CounterValue_free(cv);
	AE_CounterValue_free(tmpl);
	dlg.hide();
	return false;
      }
  
      res=dlg.fromGui(cv);
      if (res!=0) {
	DBG_ERROR(0, "Should not happen");
	dlg.fini();
	AE_CounterValue_free(cv);
	AE_CounterValue_free(tmpl);
	dlg.hide();
	return false;
      }

      /* create refueling */
      rv=AE_Book_AddCounterValue(b, AE_Book_TableType_CounterValue, cv, 1);
      if (rv<0) {
	dlg.fini();
	AE_CounterValue_free(cv);
	AE_CounterValue_free(tmpl);
	dlg.hide();
	return false;
      }
      app->signalDbReload(AE_Book_TableType_CounterValue);

      /* make current refueling the template for the next round */
      AE_CounterValue_free(tmpl);
      tmpl=cv;
      AE_CounterValue_SetId(tmpl, 0);
      AE_CounterValue_SetDbId(tmpl, 0);
      AE_CounterValue_SetValueX(tmpl, NULL);
      AE_CounterValue_SetValueY(tmpl, NULL);
      AE_CounterValue_SetValueZ(tmpl, NULL);

      if (!dlg.wantAnother()) {
	dlg.fini();
	AE_CounterValue_free(tmpl);
	dlg.hide();
	return true;
      }
  
    }
  }
  else {
    DBG_ERROR(0, "No open book");
    return false;
  }
}






