/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_tanimagedialog.hpp"
#include "ff_app.hpp"


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)





FXDEFMAP(FF_TanImageDialog) FF_TanImageDialogMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_TanImageDialog::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_TanImageDialog::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_TanImageDialog::ID_EDIT_TAN, FF_TanImageDialog::onCmdTan),
  FXMAPFUNC(SEL_CHANGED, FF_TanImageDialog::ID_EDIT_TAN, FF_TanImageDialog::onChgTan),
};


FXIMPLEMENT(FF_TanImageDialog, FXDialogBox, FF_TanImageDialogMap, ARRAYNUMBER(FF_TanImageDialogMap))




FF_TanImageDialog::FF_TanImageDialog(FF_App* a,
                                   FXWindow *parent,
                                   const FXString& title,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h,
                                   FXint pl, FXint pr, FXint pt, FXint pb,
                                   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_tanMinLen(4)
,m_tanMaxLen(6)
,m_imageWidget(NULL)
,m_tanInput(NULL)
,m_okButton(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXHorizontalFrame *hf;

  m_label=new FOX16_HtmlLabel(vf, "", LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label->setMinimumWidth(200);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    m_imageWidget=new FXImageFrame(hf, NULL, LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT);

  new FXSpring(vf, LAYOUT_FILL_X);

  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Please enter the TAN"), NULL, LABEL_NORMAL);
  m_tanInput=new FXTextField(hf, 40, this, ID_EDIT_TAN, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
                  BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton=new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
                             BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton->disable();
}



FF_TanImageDialog::FF_TanImageDialog()
:FXDialogBox()
,m_app(NULL)
,m_tanMinLen(4)
,m_tanMaxLen(6)
,m_imageWidget(NULL)
,m_tanInput(NULL)
,m_okButton(NULL)
{

}



FF_TanImageDialog::~FF_TanImageDialog() {
}



void FF_TanImageDialog::init() {
  GWEN_DB_NODE *db;
  int w, h;
  FXString str;

  db=m_app->getDialogConfig("FF_TanImageDialog");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  GWEN_DB_Group_free(db);
}



void FF_TanImageDialog::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_TanImageDialog", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);

}



long FF_TanImageDialog::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  FXString s;
  int len;

  s=m_tanInput->getText();
  len=s.length();
  if (len>=m_tanMinLen && len<m_tanMaxLen) {
    getApp()->stopModal(this, TRUE);
  }

  return 1;
}



long FF_TanImageDialog::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



long FF_TanImageDialog::onCmdTan(FXObject*, FXSelector, void *ptr) {
  FXString s;
  int len;

  s=m_tanInput->getText();
  len=s.length();
  if (len>=m_tanMinLen && len<m_tanMaxLen) {
    getApp()->stopModal(this, TRUE);
  }

  return 1;
}



long FF_TanImageDialog::onChgTan(FXObject*, FXSelector, void *ptr) {
  FXString s;
  int len;

  s=m_tanInput->getText();
  len=s.length();
  if (len>=m_tanMinLen && len<m_tanMaxLen)
    m_okButton->enable();
  else
    m_okButton->disable();

  return 1;
}



bool FF_TanImageDialog::getTan(FF_App* app,
                               FXWindow *parent,
                               const FXString &title,
                               const FXString &message,
                               FXImage *image,
                               int minLen,
                               int maxLen,
                               FXString &response) {
  FF_TanImageDialog dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.create();
  dlg.m_tanMaxLen=maxLen;
  dlg.m_tanMinLen=minLen;

  dlg.m_label->setText(message);
  dlg.init();
  dlg.layout();
  dlg.show(PLACEMENT_VISIBLE);

  dlg.m_imageWidget->setImage(image);
  dlg.m_imageWidget->resize(image->getWidth(), image->getHeight());

  dlg.m_tanInput->setFocus();
  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();

  if (dlgres==0)
    return false;

  response=dlg.m_tanInput->getText();

  return true;
}







