/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_view_report2_cnt_graph.hpp"
#include "ff_accountedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/report2/af_report2_settings.h>
#include <aqfinance/report2/af_report2.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_View_Report2CounterGraph) FF_View_Report2CounterGraphMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_View_Report2CounterGraph::ID_BUTTON_ENABLED, FF_View_Report2CounterGraph::onCmdEnable)
};


FXIMPLEMENT(FF_View_Report2CounterGraph, FXHorizontalFrame, FF_View_Report2CounterGraphMap, ARRAYNUMBER(FF_View_Report2CounterGraphMap))




FF_View_Report2CounterGraph::FF_View_Report2CounterGraph(FXComposite *p, FXuint opts, FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hfRow;
  FXGroupBox *gb;

  m_enableButton=new FXCheckButton(vf, I18N("Enabled"), this, ID_BUTTON_ENABLED,
                                   LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
  hfRow=new FXHorizontalFrame(vf, LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

  /* dimensions */
  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2CounterGraph|Data"), LAYOUT_FILL_X | FRAME_LINE);
  if (1) {
    FXVerticalFrame *vf2=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

    m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Label"), NULL, LABEL_NORMAL);
    m_labelEdit=new FXTextField(m, 1, this, ID_EDIT_LABEL,
                                TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Data Mode"), NULL, LABEL_NORMAL);
    m_dataModeCombo=new FXComboBox(m, 1, this, ID_COMBO_DATAMODE,
                                   COMBOBOX_STATIC |
                                   FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_dataModeCombo->appendItem(I18N("-- select type --"));
    m_dataModeCombo->appendItem(I18N("AF_Report2_DataMode|direct"));
    m_dataModeCombo->appendItem(I18N("AF_Report2_DataMode|average"));
    m_dataModeCombo->appendItem(I18N("AF_Report2_DataMode|floating average"));
    m_dataModeCombo->setNumVisible(m_dataModeCombo->getNumItems());

    new FXLabel(m, I18N("Sum Type"), NULL, LABEL_NORMAL);
    m_sumTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_DATAMODE,
                                  COMBOBOX_STATIC |
                                  FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_sumTypeCombo->appendItem(I18N("-- select type --"));
    m_sumTypeCombo->appendItem(I18N("AF_REPORT2_FLAGS_SUM|total"));
    m_sumTypeCombo->appendItem(I18N("AF_REPORT2_FLAGS_SUM|positive sums only"));
    m_sumTypeCombo->appendItem(I18N("AF_REPORT2_FLAGS_SUM|negative sums only"));
    m_sumTypeCombo->setNumVisible(m_sumTypeCombo->getNumItems());

    m_reverseSignButton=new FXCheckButton(vf2, I18N("Reverse sign"), this, ID_BUTTON_REVERSE_SIGN,
                                          LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
    m_accumulateButton=new FXCheckButton(vf2, I18N("Accumulate"), this, ID_BUTTON_ACCUMULATE,
                                         LAYOUT_SIDE_LEFT | CHECKBUTTON_NORMAL);
  }


  gb=new FXGroupBox(hfRow, I18N("FF_View_Report2CounterGraph|Graph"), LAYOUT_FILL_X | FRAME_LINE);
  if (1) {
    FXVerticalFrame *vf2=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

    m=new FXMatrix(vf2, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Graph Type"), NULL, LABEL_NORMAL);
    m_graphTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_GRAPHTYPE,
                                    COMBOBOX_STATIC |
                                    FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_graphTypeCombo->appendItem(I18N("-- select type --"));
    m_graphTypeCombo->appendItem(I18N("AF_GRAPH_TYPE|line"));
    m_graphTypeCombo->appendItem(I18N("AF_GRAPH_TYPE|points"));
    m_graphTypeCombo->appendItem(I18N("AF_GRAPH_TYPE|bars"));
    m_graphTypeCombo->setNumVisible(m_graphTypeCombo->getNumItems());

    new FXLabel(m, I18N("Colour"), NULL, LABEL_NORMAL);
    m_colourWell=new FXColorWell(m, 0, this, ID_COLOURWELL_COLOUR,
                                 COLORWELL_NORMAL | COLORWELL_OPAQUEONLY | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Line Width"), NULL, LABEL_NORMAL);
    m_lineWidthSpinner=new FXSpinner(m, 1, this, ID_SPINNER_LINEWIDTH, SPIN_NORMAL | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Dash Type"), NULL, LABEL_NORMAL);
    m_dashTypeCombo=new FXComboBox(m, 1, this, ID_COMBO_DASHTYPE,
                                   COMBOBOX_STATIC |
                                   FRAME_SUNKEN | FRAME_THICK | LAYOUT_FILL_COLUMN | LAYOUT_FILL_X);
    m_dashTypeCombo->appendItem(I18N("-- select type --"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|none"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|1"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|2"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|3"));
    m_dashTypeCombo->appendItem(I18N("AF_Graphics_Dash|4"));
    m_dashTypeCombo->setNumVisible(m_dashTypeCombo->getNumItems());
  }



  /* account selection */
  gb=new FXGroupBox(vf, I18N("FF_View_Report2CounterGraph|Account Selection"), LAYOUT_FILL_X | FRAME_LINE);
  if (1) {
    FXVerticalFrame *vf2=new FXVerticalFrame(gb, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, DEFAULT_SPACING, 0);

    m=new FXMatrix(vf2, 3, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

    new FXLabel(m, I18N("Counter 1"), NULL, LABEL_NORMAL);
    m_counterComboEdit1=new FXXComboEdit(m, 40, this, ID_COUNTEREDIT_1,
                                         COMBOBOX_NORMAL | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
    m_fieldComboEdit1=new FXComboBox(m, 1, this, ID_FIELDEDIT_1,
                                     COMBOBOX_NORMAL | COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
    _addFieldNames(m_fieldComboEdit1);
    m_fieldComboEdit1->setNumVisible((m_fieldComboEdit1->getNumItems()>10)?10:m_fieldComboEdit1->getNumItems());


    new FXLabel(m, I18N("Counter 2"), NULL, LABEL_NORMAL);
    m_counterComboEdit2=new FXXComboEdit(m, 40, this, ID_COUNTEREDIT_2,
                                         COMBOBOX_NORMAL | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
    m_fieldComboEdit2=new FXComboBox(m, 1, this, ID_FIELDEDIT_2,
                                     COMBOBOX_NORMAL | COMBOBOX_STATIC | LAYOUT_FILL_X | LAYOUT_FILL_COLUMN);
    _addFieldNames(m_fieldComboEdit2);
    m_fieldComboEdit2->setNumVisible((m_fieldComboEdit2->getNumItems()>10)?10:m_fieldComboEdit2->getNumItems());
  }


}



FF_View_Report2CounterGraph::FF_View_Report2CounterGraph()
:FXHorizontalFrame()
{
}



FF_View_Report2CounterGraph::~FF_View_Report2CounterGraph() {
}



void FF_View_Report2CounterGraph::_addFieldNames(FXComboBox *w) {
  w->clearItems();

  w->appendItem("valueX");
  w->appendItem("diffX");
  w->appendItem("xPerDay");
  w->appendItem("xPerMonth");
  w->appendItem("xPerYear");
  w->appendItem("totalXPerDay");
  w->appendItem("totalXPerMonth");
  w->appendItem("totalXPerYear");

  w->appendItem("valueY");
  w->appendItem("diffY");
  w->appendItem("yPerDay");
  w->appendItem("yPerMonth");
  w->appendItem("yPerYear");
  w->appendItem("totalYPerDay");
  w->appendItem("totalYPerMonth");
  w->appendItem("totalYPerYear");

  w->appendItem("valueZ");
  w->appendItem("diffZ");
  w->appendItem("zPerDay");
  w->appendItem("zPerMonth");
  w->appendItem("zPerYear");
  w->appendItem("totalZPerDay");
  w->appendItem("totalZPerMonth");
  w->appendItem("totalZPerYear");
}



int FF_View_Report2CounterGraph::_reloadCounterNames(FXXComboEdit *w, const AE_COUNTER_TYPE_LIST *counterTypeList) {
  AE_COUNTER_TYPE *ct;

  w->clearItems();

  ct=AE_CounterType_List_First(counterTypeList);
  while(ct) {
    const char *s;

    s=AE_CounterType_GetName(ct);
    if (s && *s) {
      w->addEntry(s);
    }
    else {
      FXString str;

      str="#";
      str+=FXStringVal(AE_CounterType_GetId(ct));
      w->addEntry(str.text());
    }

    ct=AE_CounterType_List_Next(ct);
  }

  return 0;
}



int FF_View_Report2CounterGraph::_setCounterCombos(const char *s, const AE_COUNTER_TYPE_LIST *counterTypeList,
                                                   FXXComboEdit *wCounterName, FXComboBox *wFieldName) {
  if (s && *s) {
    AE_COUNTER_TYPE *t=NULL;
    char *copyOfS;
    char *p;
    const char *pCounterName=NULL;
    const char *pCounterFieldName=NULL;

    copyOfS=strdup(s);
    pCounterName=copyOfS;
    p=strchr(copyOfS, ':');
    if (p) {
      *p=0;
      pCounterFieldName=p+1;
    }
  
    if (*pCounterName=='#') {
      long long unsigned int u;
  
      /* get counter type by id */
      if (1!=sscanf(pCounterName+1, "%llu", &u)) {
        DBG_ERROR(AE_LOGDOMAIN, "Invalid Counter type id \"%s\"", pCounterName+1);
        free(copyOfS);
        return GWEN_ERROR_BAD_DATA;
      }
  
      /* get counter type by id */
      t=AE_CounterType_List_GetById(counterTypeList, (AQDB_ID) u);
      if (t==NULL) {
        DBG_ERROR(AE_LOGDOMAIN, "Counter type \"%llu\" not found", u);
        free(copyOfS);
        return GWEN_ERROR_BAD_DATA;
      }
    }
    else {
      /* get counter type by name */
      t=AE_CounterType_List_GetByName(counterTypeList, pCounterName);
      if (t==NULL) {
        DBG_ERROR(AE_LOGDOMAIN, "Counter type \"%s\" not found", pCounterName);
        free(copyOfS);
        return GWEN_ERROR_BAD_DATA;
      }
    }

    /* set combos */
    if (t) {
      const char *s;

      s=AE_CounterType_GetName(t);
      if (s && *s)
        wCounterName->setText(s);
      else {
        FXString str;

        str="#";
        str+=FXStringVal(AE_CounterType_GetId(t));
        wCounterName->setText(str.text());
      }
    }

    if (pCounterFieldName && *pCounterFieldName)
      wFieldName->setText(pCounterFieldName);

    /* done */
    free(copyOfS);
  }

  return 0;
}



void FF_View_Report2CounterGraph::init(GWEN_DB_NODE *db) {
  FF_App *app;
  AE_BOOK *b;
  AE_COUNTER_TYPE_LIST *counterTypeList=NULL;
  int rv;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  assert(b);

  counterTypeList=AE_CounterType_List_new();
  rv=AE_Book_ReadCounterTypeList(b, AE_Book_TableType_CounterType, counterTypeList);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_CounterType_List_free(counterTypeList);
  }
  else {
    _reloadCounterNames(m_counterComboEdit1, counterTypeList);
    _reloadCounterNames(m_counterComboEdit2, counterTypeList);
  }
  AE_CounterType_List_free(counterTypeList);

  m_enableButton->setCheck(FALSE);
  m_graphTypeCombo->setCurrentItem(1, TRUE);
  m_sumTypeCombo->setCurrentItem(1, TRUE);
  m_dataModeCombo->setCurrentItem(1, TRUE);
  m_lineWidthSpinner->setValue(1);
  m_colourWell->setRGBA(FXRGBA(255, 0, 0, 0));
  m_dashTypeCombo->setCurrentItem(1, TRUE);
  disableInputs();
}



void FF_View_Report2CounterGraph::fini(GWEN_DB_NODE *db) {
}



void FF_View_Report2CounterGraph::toGui(const AF_REPORT2_SETTINGS_GRAPH *settings) {
  FF_App *app;
  AE_BOOK *b;
  AE_COUNTER_TYPE_LIST *counterTypeList=NULL;
  const char *s;
  int i;
  uint32_t scol;
  FXColor col;
  int rv;

  m_enableButton->setCheck((AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_ENABLED)?TRUE:FALSE);
  if (m_enableButton->getCheck())
    enableInputs();
  else
    disableInputs();

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  assert(b);

  counterTypeList=AE_CounterType_List_new();
  rv=AE_Book_ReadCounterTypeList(b, AE_Book_TableType_CounterType, counterTypeList);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_CounterType_List_free(counterTypeList);
    return;
  }

  _reloadCounterNames(m_counterComboEdit1, counterTypeList);
  _reloadCounterNames(m_counterComboEdit2, counterTypeList);


  s=AF_Report2_SettingsGraph_GetLabel(settings);
  m_labelEdit->setText(s?s:"");

  i=0;
  switch(AF_Report2_SettingsGraph_GetGraphType(settings)) {
    case AF_GRAPH_TYPE_NONE:   i=0; break;
    default:
    case AF_GRAPH_TYPE_LINE:   i=1; break;
    case AF_GRAPH_TYPE_POINTS: i=2; break;
    case AF_GRAPH_TYPE_BARS:   i=3; break;
  }
  m_graphTypeCombo->setCurrentItem(i);

  switch(AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_SUM_MASK) {
    default:
    case AF_REPORT2_FLAGS_SUM_TOTAL:    i=1; break;
    case AF_REPORT2_FLAGS_SUM_POSITIVE: i=2; break;
    case AF_REPORT2_FLAGS_SUM_NEGATIVE: i=3; break;
  }
  m_sumTypeCombo->setCurrentItem(i);

  i=0;
  switch(AF_Report2_SettingsGraph_GetDataMode(settings)) {
    default:
    case AF_Report2_DataMode_Direct:          i=1; break;
    case AF_Report2_DataMode_Average:         i=2; break;
    case AF_Report2_DataMode_FloatingAverage: i=3; break;
  }
  m_dataModeCombo->setCurrentItem(i);

  scol=AF_Report2_SettingsGraph_GetColour(settings);
  col=FXRGB((scol>>24) & 0xff, (scol>>16) & 0xff, (scol>>8) & 0xff);
  m_colourWell->setRGBA(col);

  i=AF_Report2_SettingsGraph_GetLineWidth(settings);
  if (i==0)
    i=1;
  m_lineWidthSpinner->setValue(i, TRUE);

  i=0;
  switch(AF_Report2_SettingsGraph_GetDashType(settings)) {
    default:
    case 0: i=1; break;
    case 1: i=2; break;
    case 2: i=3; break;
    case 3: i=4; break;
    case 4: i=5; break;
  }
  m_dashTypeCombo->setCurrentItem(i, TRUE);

  s=AF_Report2_SettingsGraph_GetCategoryNameAt(settings, 0);
  if (s && *s)
    _setCounterCombos(s, counterTypeList, m_counterComboEdit1, m_fieldComboEdit1);

  s=AF_Report2_SettingsGraph_GetCategoryNameAt(settings, 1);
  if (s)
    _setCounterCombos(s, counterTypeList, m_counterComboEdit2, m_fieldComboEdit2);

  if (AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_REVERSE_SIGN)
    m_reverseSignButton->setCheck(TRUE);
  else
    m_reverseSignButton->setCheck(FALSE);

  if (AF_Report2_SettingsGraph_GetFlags(settings) & AF_REPORT2_FLAGS_ACCUMULATE)
    m_accumulateButton->setCheck(TRUE);
  else
    m_accumulateButton->setCheck(FALSE);

  AE_CounterType_List_free(counterTypeList);
}



bool FF_View_Report2CounterGraph::fromGui(AF_REPORT2_SETTINGS_GRAPH *settings) {
  FXString str;
  FXString str2;
  uint32_t flags=0;
  int i;
  FXColor col;
  uint32_t scol;

  if (m_enableButton->getCheck())
    flags|=AF_REPORT2_FLAGS_ENABLED;
  str=m_labelEdit->getText().simplify();
  if (str.empty())
    AF_Report2_SettingsGraph_SetLabel(settings, NULL);
  else
    AF_Report2_SettingsGraph_SetLabel(settings, str.text());

  switch(m_graphTypeCombo->getCurrentItem()) {
    default:
    case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select graph type");
      m_graphTypeCombo->setFocus();
      return FALSE;
    case 1: i=AF_GRAPH_TYPE_LINE;   break;
    case 2: i=AF_GRAPH_TYPE_POINTS; break;
    case 3: i=AF_GRAPH_TYPE_BARS;   break;
  }
  AF_Report2_SettingsGraph_SetGraphType(settings, i);

  switch(m_sumTypeCombo->getCurrentItem()) {
    default:
    case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select sum type");
      m_sumTypeCombo->setFocus();
      return FALSE;
    case 1: i=AF_REPORT2_FLAGS_SUM_TOTAL;    break;
    case 2: i=AF_REPORT2_FLAGS_SUM_POSITIVE; break;
    case 3: i=AF_REPORT2_FLAGS_SUM_NEGATIVE; break;
  }
  flags|=i;


  switch(m_dataModeCombo->getCurrentItem()) {
  case 0:
      DBG_ERROR(AE_LOGDOMAIN, "Select data mode");
      m_dataModeCombo->setFocus();
      return FALSE;
  case 1: i=AF_Report2_DataMode_Direct;          break;
  case 2: i=AF_Report2_DataMode_Average;         break;
  case 3: i=AF_Report2_DataMode_FloatingAverage; break;
  }
  AF_Report2_SettingsGraph_SetDataMode(settings, i);


  col=m_colourWell->getRGBA();
  scol=
    (FXREDVAL(col)<<24) +
    (FXGREENVAL(col)<<16)+
    (FXBLUEVAL(col)<<8)+
    0xff;

  AF_Report2_SettingsGraph_SetColour(settings, scol);
  AF_Report2_SettingsGraph_SetLineWidth(settings, m_lineWidthSpinner->getValue());

  i=0;
  switch(m_dashTypeCombo->getCurrentItem()) {
    default:
    case 1: i=0; break;
    case 2: i=1; break;
    case 3: i=2; break;
    case 4: i=3; break;
    case 5: i=4; break;
  }
  AF_Report2_SettingsGraph_SetDashType(settings, i);

  if (m_reverseSignButton->getCheck())
    flags|=AF_REPORT2_FLAGS_REVERSE_SIGN;

  if (m_accumulateButton->getCheck())
    flags|=AF_REPORT2_FLAGS_ACCUMULATE;

  AF_Report2_SettingsGraph_SetFlags(settings, flags);


  str=m_counterComboEdit1->getText().simplify();
  str2=m_fieldComboEdit1->getText().simplify();
  if (!str.empty() && !str2.empty()) {
    FXString str3;

    str3=str+":"+str2;
    AF_Report2_SettingsGraph_AddCategoryName(settings, str3.text());
  }

  str=m_counterComboEdit2->getText().simplify();
  str2=m_fieldComboEdit2->getText().simplify();
  if (!str.empty() && !str2.empty()) {
    FXString str3;

    str3=str+":"+str2;
    AF_Report2_SettingsGraph_AddCategoryName(settings, str3.text());
  }

  DBG_ERROR(0, "fromGui(): ok");
  return true;
}



void FF_View_Report2CounterGraph::enableInputs() {
  m_labelEdit->enable();
  m_graphTypeCombo->enable();
  m_sumTypeCombo->enable();
  m_dataModeCombo->enable();

  m_colourWell->enable();
  m_lineWidthSpinner->enable();
  m_dashTypeCombo->enable();

  m_reverseSignButton->enable();
  m_accumulateButton->enable();

  m_counterComboEdit1->enable();
  m_fieldComboEdit1->enable();

  m_counterComboEdit2->enable();
  m_fieldComboEdit2->enable();
}



void FF_View_Report2CounterGraph::disableInputs() {
  m_labelEdit->disable();
  m_graphTypeCombo->disable();
  m_sumTypeCombo->disable();
  m_dataModeCombo->disable();

  m_colourWell->disable();
  m_lineWidthSpinner->disable();
  m_dashTypeCombo->disable();

  m_reverseSignButton->disable();
  m_accumulateButton->disable();

  m_counterComboEdit1->disable();
  m_fieldComboEdit1->disable();

  m_counterComboEdit2->disable();
  m_fieldComboEdit2->disable();
}



long FF_View_Report2CounterGraph::onCmdEnable(FXObject*, FXSelector, void *ptr) {
  if (m_enableButton->getCheck())
    enableInputs();
  else
    disableInputs();
  return 1;
}








