/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_bankaccaccpage.hpp"
#include "fm_bankaccpage.hpp"
#include "ff_bankaccounttree.hpp"
#include "ff_app.hpp"
#include "ff_editbankaccgroup.hpp"
#include "ff_editbankaccount.hpp"
#include "ff_selectabacc.hpp"
#include "ff_selectabaccs.hpp"
#include "ff_pickstartdate.hpp"
#include "ff_dateedit.hpp"

#include <aqfinance/engine/modules/ae_update_bankaccounts.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_BankAccountAccPage) FM_BankAccountAccPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_ONLINE1,
            FM_BankAccountAccPage::onCmdOnlineUpdate1),
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_ONLINE2,
            FM_BankAccountAccPage::onCmdOnlineUpdate2),

  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_ADDGROUP,
            FM_BankAccountAccPage::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_EDITGROUP,
            FM_BankAccountAccPage::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_DELGROUP,
            FM_BankAccountAccPage::onCmdDelGroup),

  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_ADDACCOUNT,
            FM_BankAccountAccPage::onCmdAddAccount),
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_EDITACCOUNT,
            FM_BankAccountAccPage::onCmdEditAccount),
  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_BUTTON_DELACCOUNT,
            FM_BankAccountAccPage::onCmdDelAccount),

  FXMAPFUNC(SEL_CHANGED, FM_BankAccountAccPage::ID_ACCOUNT_TREE,
            FM_BankAccountAccPage::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FM_BankAccountAccPage::ID_ACCOUNT_TREE,
            FM_BankAccountAccPage::onTreeCommand),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_BankAccountAccPage::ID_ACCOUNT_TREE,
            FM_BankAccountAccPage::onTreeRightBtnPress),

  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_ASSIGN_BANKING_ACCOUNT,
            FM_BankAccountAccPage::onCmdAssignBankingAccount),

  FXMAPFUNC(SEL_COMMAND, FM_BankAccountAccPage::ID_IMPORT_BANKING_ACCOUNTS,
            FM_BankAccountAccPage::onCmdImportBankingAccounts),

};



FXIMPLEMENT(FM_BankAccountAccPage, FXHorizontalFrame,
            FM_BankAccountAccPageMap, ARRAYNUMBER(FM_BankAccountAccPageMap))






FM_BankAccountAccPage::FM_BankAccountAccPage(FM_BankAccountPage *mainPage,
                                             FXComposite *p,
                                             FXuint opts,
                                             FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_bankAccountTree(NULL)
,m_onlineUpdateButton(NULL)
{
  FXIcon *ic;
  FF_App *app;
  FXVerticalFrame *vf;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_bankAccountTree=new FF_BankAccountTree(this, this, ID_ACCOUNT_TREE,
                                           LAYOUT_SIDE_LEFT |
                                           LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                           FOLDINGLIST_EXTENDEDSELECT |
                                           FOLDINGLIST_SHOWS_LINES |
                                           FOLDINGLIST_SHOWS_BOXES |
                                           FOLDINGLIST_ROOT_BOXES |
                                           FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_bankAccountTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_bankAccountTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);
  m_onlineUpdateButton=new THEMEBUTTON(vf,
                                       I18N("Online Update"),
                                       NULL,
                                       this, ID_BUTTON_ONLINE1,
                                       BUTTON_NORMAL);

}




FM_BankAccountAccPage::FM_BankAccountAccPage()
:FXHorizontalFrame()
,m_mainPage(NULL)
,m_bankAccountTree(NULL)
,m_onlineUpdateButton(NULL)
{
}



FM_BankAccountAccPage::~FM_BankAccountAccPage() {
}



void FM_BankAccountAccPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "accountPage");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_bankAccountTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_bankAccountTree->getHeaderSize(i));
      m_bankAccountTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_bankAccountTree->setHeaderArrowDir(i, TRUE);
        else
          m_bankAccountTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_BankAccountAccPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "accountPage");
  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_bankAccountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_bankAccountTree->getHeaderSize(i));
    b=m_bankAccountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_BankAccountAccPage::create(){
  FXHorizontalFrame::create();
}



void FM_BankAccountAccPage::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_BankAccountView: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  m_bankAccountTree->dump(indent+2);

}



bool FM_BankAccountAccPage::getLatestStatementDate(AQDB_ID aid, FXDate &res) {
  FF_App *app;
  AE_BOOK *b;
  AE_STATEMENT *latestStatement=NULL;
  int rv;
  char qexpr[256];

  snprintf(qexpr, sizeof(qexpr)-1,
	   "$bankAccountId==%d",
	   aid);
  qexpr[sizeof(qexpr)-1]=0;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  rv=AE_Book_FindLatestStatement(b, qexpr, &latestStatement);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  else if (latestStatement) {
    const GWEN_DATE *d;

    d=AE_Statement_GetDate(latestStatement);
    if (d) {
      res=FXDate(GWEN_Date_GetYear(d),
		 GWEN_Date_GetMonth(d),
		 GWEN_Date_GetDay(d));
      AE_Statement_free(latestStatement);
      return true;
    }
    AE_Statement_free(latestStatement);
  }

  return false;
}



void FM_BankAccountAccPage::createOnlineUpdateJobs(FXbool letUserPickDate) {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;
  bool jobsAdded=false;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (ab && b) {
    AE_BANKACCOUNT_LIST2 *al;
    AE_BANKACCOUNT_LIST2_ITERATOR *it;

    al=AE_BankAccount_List2_new();
    m_bankAccountTree->getSelectedBankAccounts(al);
    it=AE_BankAccount_List2_First(al);
    if (it) {
      AB_ACCOUNT_SPEC_LIST *asl=NULL;
      int rv;

      rv=AB_Banking_GetAccountSpecList(ab, &asl);
      if (rv<0) {
        DBG_ERROR(0, "No account specs available (%d)", rv);
      }
      else {
        AE_BANKACCOUNT *ba;

        ba=AE_BankAccount_List2Iterator_Data(it);
        while (ba) {
          AQDB_ID bankingId;

          bankingId=AE_BankAccount_GetBankingId(ba);
          if (bankingId) {
            AB_ACCOUNT_SPEC *as;

            as=AB_AccountSpec_List_GetByUniqueId(asl, (uint32_t)bankingId);
            if (as) {
              AB_TRANSACTION *j;
              AB_TRANSACTION_LIMITS *lim;

              if (!queueHasJobForAccount(bankingId, AB_Transaction_CommandGetTransactions)) {
                j=AB_Transaction_new();
                lim=AB_AccountSpec_GetTransactionLimitsForCommand(as, AB_Transaction_CommandGetTransactions);
                if (lim==NULL) {
                  DBG_ERROR(0, "Job is not available");
                  AB_Transaction_free(j);
                }
                else {
                  FXDate startDate;

                  AB_Transaction_SetCommand(j, AB_Transaction_CommandGetTransactions);
                  AB_Transaction_SetUniqueAccountId(j, bankingId);
                  AB_Banking_FillTransactionFromAccountSpec(j, as);

                  if (letUserPickDate) {
                    FXDate d;
                    FXDate firstPossible=FXDate(1800, 1, 1);
                    FXDate lastUpdate=FXDate(1800, 1, 1);
      
                    getLatestStatementDate(AE_BankAccount_GetId(ba), lastUpdate);
      
                    if (!FF_PickStartDate::pickStartDate(this, I18N("Pick Start Date"), firstPossible, lastUpdate, 0, d))
                      break;

                    if (d>=FXDate(1900, 1, 1) && d<FXDate(2500, 12, 31)) {
                      GWEN_DATE *da=NULL;
      
                      if (!FF_DateEdit::convertDate(d, &da)) {
                        DBG_ERROR(0, "Bad date");
                        AB_Transaction_free(j);
                        break;
                      }
                      AB_Transaction_SetFirstDate(j, da);
                      GWEN_Date_free(da);
                    }
                  }
                  else {
                    if (getLatestStatementDate(AE_BankAccount_GetId(ba), startDate)) {
                      GWEN_DATE *da;

                      da=GWEN_Date_fromGregorian(startDate.year(), startDate.month(), startDate.day());
                      if (da) {
                        GWEN_DATE *today;

                        today=GWEN_Date_CurrentDate();
                        if (GWEN_Date_Compare(today, da)<0)
                          AB_Transaction_SetFirstDate(j, today);
                        else
                          AB_Transaction_SetFirstDate(j, da);
                        GWEN_Date_free(today);
                        GWEN_Date_free(da);
                      } /* if (da) */
                    } /* if (getLatestStatementDate(AE_BankAccount_GetId(ba), startDate)) */
                  }

                  app->addJobToList(j);
                  jobsAdded=true;
                } /* if lim */
              } /* if (!queueHasJobForAccount(bankingId, AB_Transaction_CommandGetTransactions)) */
              else {
                DBG_INFO(0, "There already is a job of this type for this account in the queue, not adding");
              }


              if (!queueHasJobForAccount(bankingId, AB_Transaction_CommandGetBalance)) {
                j=AB_Transaction_new();
                lim=AB_AccountSpec_GetTransactionLimitsForCommand(as, AB_Transaction_CommandGetBalance);
                if (lim==NULL) {
                  DBG_ERROR(0, "Job is not available (%d)", rv);
                  AB_Transaction_free(j);
                }
                else {
                  AB_Transaction_SetCommand(j, AB_Transaction_CommandGetBalance);
                  AB_Transaction_SetUniqueAccountId(j, bankingId);
                  AB_Banking_FillTransactionFromAccountSpec(j, as);

                  app->addJobToList(j);
                  jobsAdded=true;
                } /* if lim */
              } /* if (!queueHasJobForAccount(bankingId, AB_Transaction_CommandGetBalance)) */
              else {
                DBG_INFO(0, "There already is a job of this type for this account in the queue, not adding");
              }

#if 0
              if (!queueHasJobForAccount(bankingId, AB_Transaction_CommandSepaGetStandingOrders)) {
                j=AB_Transaction_new();
                lim=AB_AccountSpec_GetTransactionLimitsForCommand(as, AB_Transaction_CommandSepaGetStandingOrders);
                if (lim==NULL) {
                  DBG_ERROR(0, "Job is not available (%d)", rv);
                  AB_Transaction_free(j);
                }
                else {
                  AB_Transaction_SetCommand(j, AB_Transaction_CommandGetTransactions);
                  AB_Transaction_SetUniqueAccountId(j, bankingId);
                  AB_Banking_FillTransactionFromAccountSpec(j, as);

                  app->addJobToList(j);
                  jobsAdded=true;
                } /* if lim */
              } /* if (!queueHasJobForAccount(bankingId, AB_Transaction_CommandGetTransactions)) */
              else {
                DBG_INFO(0, "There already is a job of this type for this account in the queue, not adding");
              }
#endif

            } /* if as */
          } /* if (bankingId) */
          ba=AE_BankAccount_List2Iterator_Next(it);
        } /* while */
        AB_AccountSpec_List_free(asl);
      } /* if account specs */
    } /* if (it) */
    AE_BankAccount_List2_free(al);
  } /* if (ab && b) */

  if (jobsAdded) {
    m_bankAccountTree->killSelection();
    app->signalJobListUpdated();
  }
}



long FM_BankAccountAccPage::onCmdOnlineUpdate1(FXObject*, FXSelector, void*) {
  createOnlineUpdateJobs(FALSE);
  return 1;
}



long FM_BankAccountAccPage::onCmdOnlineUpdate2(FXObject*, FXSelector, void*) {
  createOnlineUpdateJobs(TRUE);
  return 1;
}



long FM_BankAccountAccPage::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_bankAccountTree->getCurrentGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    if (FF_EditBankAccountGroup::editGroup(app, this,
                                           I18N("Add Bank Account Group"),
                                           gr)) {
      int rv;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_BankAccountGroup, gr, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add group (%d)", rv);
      }
      app->signalDbReload(AE_Book_TableType_BankAccountGroup);
    }
    AE_Group_free(gr);
  }

  return 1;
}



long FM_BankAccountAccPage::onCmdEditGroup(FXObject*, FXSelector, void*) {
  AE_GROUP *gr;

  gr=m_bankAccountTree->getCurrentGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditBankAccountGroup::editGroup(app, this,
                                             I18N("Edit Bank Account Group"),
                                             gr)) {
        int rv;

        rv=AE_Book_WriteGroup(b, AE_Book_TableType_BankAccountGroup, gr, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to edit group (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_BankAccountGroup);
      }
    }
  }
  return 1;
}



long FM_BankAccountAccPage::onCmdDelGroup(FXObject*, FXSelector, void*) {
  return 1;
}



long FM_BankAccountAccPage::onCmdAddAccount(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_BANKACCOUNT *ba;
    AE_GROUP *gr;

    ba=AE_BankAccount_new();
    gr=m_bankAccountTree->getCurrentGroup();
    if (gr)
      AE_BankAccount_SetGroupId(ba, AE_Group_GetId(gr));
    else
      AE_BankAccount_SetGroupId(ba, 0);
    if (FF_EditBankAccount::editBankAccount(app, this,
					    I18N("Add Bank Account"),
                                            DECOR_TITLE|DECOR_BORDER |
					    FF_EditBankAccount::FF_EDITBANKACCOUNT_OPTS_SHOWGROUP,
					    ba)) {
      int rv;

      rv=AE_Book_AddBankAccount(b, AE_Book_TableType_BankAccount, ba, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add bank account (%d)", rv);
      }
      app->signalDbReload(AE_Book_TableType_BankAccount);
    }
    AE_BankAccount_free(ba);
  }

  return 1;
}



long FM_BankAccountAccPage::onCmdEditAccount(FXObject*, FXSelector, void*) {
  AE_BANKACCOUNT *ba;

  ba=m_bankAccountTree->getCurrentBankAccount();
  if (ba) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditBankAccount::editBankAccount(app, this,
					      I18N("Edit Bank Account"),
					      DECOR_TITLE|DECOR_BORDER |
					      FF_EditBankAccount::FF_EDITBANKACCOUNT_OPTS_SHOWGROUP,
					      ba)) {
        int rv;

        rv=AE_Book_WriteBankAccount(b, AE_Book_TableType_BankAccount, ba, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to add bank account (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_BankAccount);
      }
    }
  }
  return 1;
}



long FM_BankAccountAccPage::onCmdDelAccount(FXObject*, FXSelector, void*) {
  AE_BANKACCOUNT *ba;

  ba=m_bankAccountTree->getCurrentBankAccount();
  if (ba) {
    FF_App *app;
    AQDB_ID id;

    app=dynamic_cast<FF_App*>(getApp());
    id=AE_BankAccount_GetId(ba);
    if (id)
      app->deleteBankAccount(id);
    app->signalDbReload(AE_Book_TableType_BankAccount);
  }
  return 1;
}



long FM_BankAccountAccPage::onCmdAssignBankingAccount(FXObject*, FXSelector, void* ptr) {
  AE_BANKACCOUNT *ba;

  ba=m_bankAccountTree->getCurrentBankAccount();
  if (ba) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AQDB_ID id;

      id=AE_BankAccount_GetBankingId(ba);
      if (FF_SelectAbAccount::selectAccount(app, this,
                                            I18N("Assign Online Banking Account"),
                                            id)) {
        int rv;

        AE_BankAccount_SetBankingId(ba, id);
        rv=AE_Book_WriteBankAccount(b, AE_Book_TableType_BankAccount, ba, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to add bank account (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_BankAccount);
      }
    }
  }
  return 1;
}



long FM_BankAccountAccPage::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
#if 0
        FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(item);
        AE_BANKACCOUNT *ba=baItem->getBankAccount();
#endif
        // TODO: So something with this bank account
      }
    }
  }
  return 1;
}



long FM_BankAccountAccPage::onTreeCommand(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item && item->isDataItem()) {
      FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(item);
      if (baItem) {
        m_mainPage->openAccountPage(baItem->getBankAccount());
      }
    }
  }

  return 1;
}



long FM_BankAccountAccPage::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_GROUP *gr;
  AE_BANKACCOUNT *ba;

  gr=m_bankAccountTree->getCurrentGroup();
  ba=m_bankAccountTree->getCurrentBankAccount();
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    new FXMenuCommand(&pmenu, I18N("Add Group"), NULL, this, ID_BUTTON_ADDGROUP);
    new FXMenuCommand(&pmenu, I18N("Add Account"), NULL, this, ID_BUTTON_ADDACCOUNT);

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Group"), NULL, this, ID_BUTTON_EDITGROUP);
    if (!gr) c->disable();

#if 0
    c=new FXMenuCommand(&pmenu, I18N("Delete Group"), NULL, this, ID_BUTTON_DELGROUP);
    if (!gr) c->disable();
#endif

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Account"), NULL, this, ID_BUTTON_EDITACCOUNT);
    if (!ba) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Account"), NULL, this, ID_BUTTON_DELACCOUNT);
    if (!ba) c->disable();

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Assign Online Account"), NULL,
                        this, ID_ASSIGN_BANKING_ACCOUNT);
    if (!ba) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Import Online Accounts"), NULL,
                        this, ID_IMPORT_BANKING_ACCOUNTS);

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Online Update"), NULL,
                        this, ID_BUTTON_ONLINE1);
    if (!ba || AE_BankAccount_GetBankingId(ba)==0) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Online Update (with date)"), NULL,
                        this, ID_BUTTON_ONLINE2);
    if (!ba || AE_BankAccount_GetBankingId(ba)==0) c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



void FM_BankAccountAccPage::dbOpened(AE_BOOK *bk) {
  dbReload(AE_Book_TableType_None);
}



void FM_BankAccountAccPage::dbClosing() {
  m_bankAccountTree->clearItems();
}



void FM_BankAccountAccPage::dbClosed() {
}



void FM_BankAccountAccPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_BankAccountGroup ||
      tt==AE_Book_TableType_BankAccount ||
      tt==AE_Book_TableType_NotedBalance ||
      tt==AE_Book_TableType_BookedBalance ||
      tt==AE_Book_TableType_None) {
    FF_App *app;
//    AB_BANKING *ab;
    AE_BOOK *b;
    AE_GROUP *gr;
    AE_BANKACCOUNT *ba;
    AQDB_ID currentAccId=0;
    AQDB_ID currentGrId=0;

    app=dynamic_cast<FF_App*>(getApp());
//    ab=app->getBanking();
    b=app->getBook();

#if 0
    /* update bank accounts before reloading */
    if (b && ab) {
      int rv;

      rv=AE_Book_UpdateBankAccounts(b, ab);
      if (rv<0) {
        DBG_INFO(0, "here (%d)", rv);
      }
    }
#endif

    gr=m_bankAccountTree->getCurrentGroup();
    if (gr)
      currentGrId=AE_Group_GetId(gr);
    ba=m_bankAccountTree->getCurrentBankAccount();
    if (ba)
      currentAccId=AE_BankAccount_GetId(ba);
  
    m_bankAccountTree->clearItems();
    if (b) {
      AE_GROUP_TREE *gt;
      int rv;
      AE_BANKACCOUNT_LIST *bal;
      AE_BANKBALANCE_LIST *notedList=NULL;
      AE_BANKBALANCE_LIST *bookedList=NULL;
      int i;

      gt=AE_Group_Tree_new();
      rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_BankAccountGroup, gt);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	m_bankAccountTree->addGroups(gt);
      }
      AE_Group_Tree_free(gt);

      /* read list of noted balances */
      notedList=AE_BankBalance_List_new();
      rv=AE_Book_ReadBankBalanceList(b, AE_Book_TableType_NotedBalance, notedList);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_BankBalance_List_free(notedList);
	notedList=NULL;
      }

      /* read list of booked balances */
      bookedList=AE_BankBalance_List_new();
      rv=AE_Book_ReadBankBalanceList(b, AE_Book_TableType_BookedBalance, bookedList);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_BankBalance_List_free(bookedList);
	bookedList=NULL;
      }

      /* read accounts */
      bal=AE_BankAccount_List_new();
      rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, bal);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	AE_BANKACCOUNT *ba;
  
	while ( (ba=AE_BankAccount_List_First(bal)) ) {
	  AE_BankAccount_List_Del(ba);

	  if (AE_BankAccount_GetOwnerType(ba)==0 &&
	      AE_BankAccount_GetOwnerId(ba)==0) {
	    AE_BANKBALANCE *bb=NULL;
	    AE_BANKBALANCE *bn=NULL;

	    /* search for booked balance */
	    if (bookedList) {
	      bb=AE_BankBalance_List_First(bookedList);
	      while(bb) {
		if (AE_BankBalance_GetAccountId(bb)==AE_BankAccount_GetId(ba))
		  break;
		bb=AE_BankBalance_List_Next(bb);
	      }
            }
            if (bb)
              AE_BankBalance_List_Del(bb);

	    /* search for noted balance */
	    if (notedList) {
	      bn=AE_BankBalance_List_First(notedList);
	      while(bn) {
		if (AE_BankBalance_GetAccountId(bn)==AE_BankAccount_GetId(ba))
		  break;
		bn=AE_BankBalance_List_Next(bn);
	      }
	    }
            if (bn)
              AE_BankBalance_List_Del(bn);

	    /* add account */
	    m_bankAccountTree->addAccount(ba, bb, bn);
	  }
	  else
	    AE_BankAccount_free(ba);
	}
      }
      AE_BankAccount_List_free(bal);
      AE_BankBalance_List_free(bookedList);
      AE_BankBalance_List_free(notedList);

      for (i=0; i<m_bankAccountTree->getNumHeaders(); i++) {
	FXbool b;
    
	b=m_bankAccountTree->getHeaderArrowDir(i);
	if (b!=MAYBE) {
	  if (b==TRUE)
	    m_bankAccountTree->sortByColumn(i, true);
	  else
	    m_bankAccountTree->sortByColumn(i, false);
	  break;
	}
      }

      m_bankAccountTree->expandItems();
    }

    if (currentAccId!=0) {
      FF_BankAccountTreeItem *item=m_bankAccountTree->findBankAccountItem(currentAccId);
      if (item) {
	item->setSelected(true);
	m_bankAccountTree->makeItemVisible(item);
	m_bankAccountTree->setCurrentItem(item);
      }
      else {
	DBG_ERROR(0, "Item not found");
      }
    }
    else if (currentGrId!=0) {
      FF_GroupTreeItem *item=m_bankAccountTree->findGroupItem(currentGrId);
      if (item) {
	item->setSelected(true);
	m_bankAccountTree->makeItemVisible(item);
	m_bankAccountTree->setCurrentItem(item);
      }
      else {
	DBG_ERROR(0, "Item not found");
      }
    }
  }
}



void FM_BankAccountAccPage::aboutToSendJobs() {
}



void FM_BankAccountAccPage::jobListUpdated() {
  FXFoldingItem *item;
  AB_TRANSACTION_LIST *jl;
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());
  jl=app->getJobList();

  item=m_bankAccountTree->getFirstItem();
  while (item) {
    FF_BankAccountTreeItem *baItem;

    baItem=dynamic_cast<FF_BankAccountTreeItem*>(item);
    if (baItem) {
      AE_BANKACCOUNT *ba;

      ba=baItem->getBankAccount();
      if (ba) {
        uint32_t aid;

        aid=AE_BankAccount_GetBankingId(ba);
        if (aid) {
          uint32_t flags;
          AB_TRANSACTION *j;

          flags=baItem->getFlags();
          flags&=~(FF_BankAccountTreeItem::FLAGS_PAYMENTS_ENQUEUED | FF_BankAccountTreeItem::FLAGS_RETRIEVAL_ENQUEUED);
          // update flags of account entries according to list
          j=AB_Transaction_List_First(jl);
          while(j) {
            if (AB_Transaction_GetStatus(j)<=AB_Transaction_StatusNone) {
              if (AB_Transaction_GetUniqueAccountId(j)==aid) {
                switch(AB_Transaction_GetCommand(j)) {
                case AB_Transaction_CommandGetBalance:
                case AB_Transaction_CommandGetTransactions:
                  flags|=FF_BankAccountTreeItem::FLAGS_RETRIEVAL_ENQUEUED;
                  break;

                case AB_Transaction_CommandTransfer:
                case AB_Transaction_CommandDebitNote:
                case AB_Transaction_CommandCreateStandingOrder:
                case AB_Transaction_CommandCreateDatedTransfer:
                case AB_Transaction_CommandInternalTransfer:
                case AB_Transaction_CommandSepaTransfer:
                case AB_Transaction_CommandSepaDebitNote:
                case AB_Transaction_CommandLoadCellPhone: /* TODO: use phone symbol */
                  flags|=FF_BankAccountTreeItem::FLAGS_PAYMENTS_ENQUEUED;
                  break;

                default:
                  break;
                }
              }
            } /* (AB_Transaction_GetStatus(j)==AB_Transaction_StatusNone) */
            j=AB_Transaction_List_Next(j);
          } /* while */

          baItem->setFlags(flags);
          m_bankAccountTree->updateItem(baItem);
        } /* if (aid) */
      } /* if (ba) */
    } /* if (baItem) */
    item=item->getBelow();
  } /* while (item) */
}



void FM_BankAccountAccPage::enter() {
}



void FM_BankAccountAccPage::leave() {
}



void FM_BankAccountAccPage::reset() {
}



long FM_BankAccountAccPage::onCmdImportBankingAccounts(FXObject*, FXSelector, void* ptr) {
  FF_App *app;
  AB_BANKING *ab;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  ab=app->getBanking();
  b=app->getBook();
  if (ab && b) {
    AB_ACCOUNT_SPEC_LIST *accountSpecList;

    accountSpecList=_getUnassignedAccountSpecList();
    if (accountSpecList) {
      AE_BANKACCOUNT_LIST *bankAccountList;

      bankAccountList=FF_SelectAbAccounts::selectAccounts(app, this,
                                                          I18N("Select AqBanking Accounts to Import"),
                                                          I18N("<p>This is the list of bank accounts known to <b>AqBanking</b>, "
                                                               "the online banking library. Please select the accounts to import.</p>"),
                                                          accountSpecList);
      AB_AccountSpec_List_free(accountSpecList);
      if (bankAccountList) {
        AE_BANKACCOUNT *ba;
        int rv;

        rv=AE_Book_BeginEdit(b, 0);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AE_BankAccount_List_free(bankAccountList);
          return 1;
        }

        ba=AE_BankAccount_List_First(bankAccountList);
        while(ba) {
          const char *sIban;
          const char *sAccountNumber;
          AQDB_ID bankingId;

          bankingId=AE_BankAccount_GetBankingId(ba);
          sIban=AE_BankAccount_GetIban(ba);
          sAccountNumber=AE_BankAccount_GetAccountNumber(ba);

          DBG_INFO(0, "Adding bank account %llu (IBAN: %s, account number: %s)",
                   (unsigned long long int) bankingId,
                   (sIban && *sIban)?sIban:"<empty>",
                   (sAccountNumber && *sAccountNumber)?sAccountNumber:"<empty>");

          rv=AE_Book_AddBankAccount(b, AE_Book_TableType_BankAccount, ba, 0);
          if (rv<0) {
            DBG_ERROR(0, "Error adding bank account %llu (IBAN: %s, account number: %s): %d",
                      (unsigned long long int) bankingId,
                      (sIban && *sIban)?sIban:"<empty>",
                      (sAccountNumber && *sAccountNumber)?sAccountNumber:"<empty>",
                      rv);
            AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
            FXMessageBox::error(this, MBOX_OK,
                                I18N("Input Error"),
                                I18N("Error adding bank account %llu (IBAN: %s, account number: %s): %d"),
                                (unsigned long long int) bankingId,
                                (sIban && *sIban)?sIban:"<empty>",
                                (sAccountNumber && *sAccountNumber)?sAccountNumber:"<empty>",
                                rv);
            AE_BankAccount_List_free(bankAccountList);
            return 1;
          }

          ba=AE_BankAccount_List_Next(ba);
        }

        AE_BankAccount_List_free(bankAccountList);

        rv=AE_Book_EndEdit(b, 0);
        if (rv<0) {
          DBG_ERROR(0, "here (%d)", rv);
          AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
          return 1;
        }

        app->signalDbReload(AE_Book_TableType_BankAccount);
      } /* if (bankAccountList) */
    } /* if (accountSpecList) */
  } /* if (ab && b) */
  return 1;
}



AB_ACCOUNT_SPEC_LIST *FM_BankAccountAccPage::_getUnassignedAccountSpecList() {
  FF_App *app;
  AB_BANKING *ab;
  AE_BOOK *b;
  int rv;

  app=dynamic_cast<FF_App*>(getApp());
  ab=app->getBanking();
  b=app->getBook();
  if (ab && b) {
    AB_ACCOUNT_SPEC_LIST *accountSpecList=NULL;
    AE_BANKACCOUNT_LIST *bankAccountList;
    AE_BANKACCOUNT *ba;

    /* read account spec list */
    rv=AB_Banking_GetAccountSpecList(app->getBanking(), &accountSpecList);
    if (rv<0) {
      DBG_ERROR(0, "No data");
      return NULL;
    }

    /* read known bank account list */
    bankAccountList=AE_BankAccount_List_new();
    rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, bankAccountList);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AB_AccountSpec_List_free(accountSpecList);
      AE_BankAccount_List_free(bankAccountList);
      return NULL;
    }

    /* remove known accounts from account spec list */
    ba=AE_BankAccount_List_First(bankAccountList);
    while(ba) {
      AQDB_ID id;

      id=AE_BankAccount_GetBankingId(ba);
      if (id) {
        AB_ACCOUNT_SPEC *as;

        as=AB_AccountSpec_List_GetByUniqueId(accountSpecList, (uint32_t) id);
        if (as) {
          /* known account, remove from list */
          AB_AccountSpec_List_Del(as);
          AB_AccountSpec_free(as);
        }
      }
      ba=AE_BankAccount_List_Next(ba);
    }
    AE_BankAccount_List_free(bankAccountList);

    if (AB_AccountSpec_List_GetCount(accountSpecList)==0) {
      AB_AccountSpec_List_free(accountSpecList);
      return NULL;
    }
    return accountSpecList;
  }

  return NULL;
}




bool FM_BankAccountAccPage::queueHasJobForAccount(uint32_t uid, AB_TRANSACTION_COMMAND jt) {
  FF_App *app;
  AB_TRANSACTION_LIST *jl;

  app=dynamic_cast<FF_App*>(getApp());
  jl=app->getJobList();
  if (jl) {
    AB_TRANSACTION *j;

    j=AB_Transaction_List_First(jl);
    while(j) {
      if (AB_Transaction_GetStatus(j)==AB_Transaction_StatusNone &&
          AB_Transaction_GetCommand(j)==jt &&
          AB_Transaction_GetUniqueAccountId(j)==uid) {
        return TRUE;
      }

      j=AB_Transaction_List_Next(j);
    } /* while */
  }

  return FALSE;
}




