/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_countersctab.hpp"
#include "fm_counterspage.hpp"
#include "ff_countertypetree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_editcountertype.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_CountersCounterTab) FM_CountersCounterTabMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_CountersCounterTab::ID_BUTTON_ADDGROUP,
            FM_CountersCounterTab::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FM_CountersCounterTab::ID_BUTTON_EDITGROUP,
            FM_CountersCounterTab::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FM_CountersCounterTab::ID_BUTTON_DELGROUP,
            FM_CountersCounterTab::onCmdDelGroup),

  FXMAPFUNC(SEL_COMMAND, FM_CountersCounterTab::ID_BUTTON_ADDCOUNTER,
            FM_CountersCounterTab::onCmdAddCounter),
  FXMAPFUNC(SEL_COMMAND, FM_CountersCounterTab::ID_BUTTON_EDITCOUNTER,
            FM_CountersCounterTab::onCmdEditCounter),
  FXMAPFUNC(SEL_COMMAND, FM_CountersCounterTab::ID_BUTTON_DELCOUNTER,
            FM_CountersCounterTab::onCmdDelCounter),

  FXMAPFUNC(SEL_CHANGED, FM_CountersCounterTab::ID_COUNTER_TREE,
            FM_CountersCounterTab::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FM_CountersCounterTab::ID_COUNTER_TREE,
            FM_CountersCounterTab::onTreeCommand),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_CountersCounterTab::ID_COUNTER_TREE,
            FM_CountersCounterTab::onTreeRightBtnPress),

};



FXIMPLEMENT(FM_CountersCounterTab, FF_ModTabPage,
            FM_CountersCounterTabMap, ARRAYNUMBER(FM_CountersCounterTabMap))






FM_CountersCounterTab::FM_CountersCounterTab(const FXString &pageId,
                                             FM_CountersPage *mainPage,
                                             FXuint opts,
                                             FXint x, FXint y, FXint w, FXint h)
:FF_ModTabPage(pageId, mainPage, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_counterTree(NULL)
,m_isEditing(false)
{
  FXIcon *ic;
  FF_App *app;

  m_closeable=false;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_counterTree=new FF_CounterTypeTree(this, this, ID_COUNTER_TREE,
                                       LAYOUT_SIDE_LEFT |
                                       LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                       FOLDINGLIST_EXTENDEDSELECT |
                                       FOLDINGLIST_SHOWS_LINES |
                                       FOLDINGLIST_SHOWS_BOXES |
                                       FOLDINGLIST_ROOT_BOXES |
                                       FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_counterTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_counterTree->setIconClosed(ic);

}




FM_CountersCounterTab::FM_CountersCounterTab()
:FF_ModTabPage()
,m_mainPage(NULL)
,m_counterTree(NULL)
,m_isEditing(false)
{
}



FM_CountersCounterTab::~FM_CountersCounterTab() {
}



void FM_CountersCounterTab::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");

  m_counterTree->addColumn(FF_CounterTypeTree::COLUMN_NAME, 100);
  m_counterTree->addColumn(FF_CounterTypeTree::COLUMN_XLABEL, 100);
  m_counterTree->addColumn(FF_CounterTypeTree::COLUMN_YLABEL, 100);
  m_counterTree->addColumn(FF_CounterTypeTree::COLUMN_ZLABEL, 100);

  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "countersTab");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_counterTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_counterTree->getHeaderSize(i));
      m_counterTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_counterTree->setHeaderArrowDir(i, TRUE);
        else
          m_counterTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_CountersCounterTab::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "countersTab");
  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_counterTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_counterTree->getHeaderSize(i));
    b=m_counterTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_CountersCounterTab::create(){
  FF_ModTabPage::create();
  m_counterTree->create();
}



void FM_CountersCounterTab::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_CounterView: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  m_counterTree->dump(indent+2);
}



long FM_CountersCounterTab::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_counterTree->getFirstSelectedGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    m_isEditing=true;
    if (FF_EditGroup::editGroup(app,
				this,
                                I18N("Add Counter Group"),
                                AE_Book_TableType_CounterTypeGroup,
                                gr)) {
      int rv;
      FF_GroupTreeItem *item;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_CounterTypeGroup, gr, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add group (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Unable to write object to database: %d"), rv);
      }
      m_counterTree->addGroup(gr);
      item=m_counterTree->findGroupItem(gr);
      assert(item);
      m_counterTree->makeItemVisible(item);
      m_counterTree->unselectAll();
      item->setSelected(TRUE);
      m_counterTree->setCurrentItem(item, TRUE);
      m_isEditing=false;
      app->signalDbReload(AE_Book_TableType_CounterTypeGroup);
    }
    else
      AE_Group_free(gr);
    m_isEditing=false;
  }

  return 1;
}



long FM_CountersCounterTab::onCmdEditGroup(FXObject*, FXSelector, void*) {
  AE_GROUP *gr;

  gr=m_counterTree->getFirstSelectedGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      m_isEditing=true;
      if (FF_EditGroup::editGroup(app,
                                  this,
                                  I18N("Edit Counter Group"),
                                  AE_Book_TableType_CounterTypeGroup,
                                  gr)) {
        int rv;
        FXString t;

        rv=AE_Book_WriteGroup(b, AE_Book_TableType_CounterTypeGroup, gr, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to edit group (%d)", rv);
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Database Error"),
                              I18N("Unable to write object to database: %d"), rv);
        }

        m_isEditing=false;
        app->signalDbReload(AE_Book_TableType_CounterTypeGroup);
      }
      m_isEditing=false;
    }
  }
  return 1;
}



long FM_CountersCounterTab::onCmdDelGroup(FXObject*, FXSelector, void*) {
#if 0
  AE_GROUP *gr;

  gr=m_counterTree->getFirstSelectedGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXuint res;

      res=FXMessageBox::question(this, MBOX_YES_NO,
                                 I18N("Delete Group"),
                                 "%s",
                                 I18N("Are you sure you want to delete the selected "
                                      "group?"));
      if (res!=MBOX_CLICKED_YES) {
        DBG_ERROR(0, "Aborted");
      }
      else {
        app->deleteCounterGroup(AE_Group_GetId(gr));
        app->signalDbReload(AE_Book_TableType_CounterTypeGroup);
        app->signalDbReload(AE_Book_TableType_CounterType);
      }
    }
  }
#else
# warning Need to implemente this!
  FXMessageBox::error(this, MBOX_OK,
                      I18N("Not Implemented"),
                      "%s",
                      I18N("The function your are calling is not yet implemented."));
#endif

  return 1;
}



long FM_CountersCounterTab::onCmdAddCounter(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_COUNTER_TYPE *ct;
    AE_GROUP *gr;

    ct=AE_CounterType_new();
    gr=m_counterTree->getFirstSelectedGroup();
    if (gr)
      AE_CounterType_SetGroupId(ct, AE_Group_GetId(gr));
    else
      AE_CounterType_SetGroupId(ct, 0);
    if (FF_EditCounterType::editCounterType(app, this,
                                            I18N("Add Counter Type"),
                                            ct)) {
      int rv;

      rv=AE_Book_AddCounterType(b, AE_Book_TableType_CounterType, ct, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add counter (%d)", rv);
      }
      app->signalDbReload(AE_Book_TableType_CounterType);
    }
    AE_CounterType_free(ct);
  }

  return 1;
}



long FM_CountersCounterTab::onCmdEditCounter(FXObject*, FXSelector, void*) {
  AE_COUNTER_TYPE *ct;

  ct=m_counterTree->getFirstSelectedObject();
  if (ct) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditCounterType::editCounterType(app, this,
                                              I18N("Edit Counter Type"),
                                              ct)) {
        int rv;

        rv=AE_Book_WriteCounterType(b, AE_Book_TableType_CounterType, ct, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to write counter back (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_CounterType);
      }
    }
  }

  return 1;
}



long FM_CountersCounterTab::onCmdDelCounter(FXObject*, FXSelector, void*) {
#if 0
  AE_COUNTER_TYPE *ct;

  ct=m_counterTree->getFirstSelectedObject();
  if (ct) {
    FF_App *app;
    AQDB_ID id;

    app=dynamic_cast<FF_App*>(getApp());
    id=AE_CounterType_GetId(ct);
    if (id)
      app->deleteCounter(id);
    app->signalDbReload(AE_Book_TableType_CounterType);
  }

#else
# warning Need to implemente this!
#endif

  return 1;
}



long FM_CountersCounterTab::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        //FF_CounterTypeTreeItem *baItem=dynamic_cast<FF_CounterTypeTreeItem*>(item);
        //AE_COUNTER_TYPE *ct=baItem->getCounter();

      }
    }
  }
  return 1;
}



long FM_CountersCounterTab::onTreeCommand(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item && item->isDataItem()) {
      FF_CounterTypeTreeItem *baItem=dynamic_cast<FF_CounterTypeTreeItem*>(item);
      if (baItem) {
        m_mainPage->openCounterPage(baItem->getObject());
      }
    }
  }
  return 1;
}



long FM_CountersCounterTab::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_GROUP *gr;
  AE_COUNTER_TYPE *ct;

  gr=m_counterTree->getCurrentGroup();
  ct=m_counterTree->getCurrentObject();
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    new FXMenuCommand(&pmenu, I18N("Add Group"), NULL, this, ID_BUTTON_ADDGROUP);
    new FXMenuCommand(&pmenu, I18N("Add Counter"), NULL, this, ID_BUTTON_ADDCOUNTER);

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Group"), NULL, this, ID_BUTTON_EDITGROUP);
    if (!gr) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Edit Group"), NULL, this, ID_BUTTON_EDITGROUP);
    if (!gr) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Group"), NULL, this, ID_BUTTON_DELGROUP);
    if (!gr) c->disable();

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Counter"), NULL, this, ID_BUTTON_EDITCOUNTER);
    if (!ct) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Counter"), NULL, this, ID_BUTTON_DELCOUNTER);
    if (!ct) c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



void FM_CountersCounterTab::dbOpened(AE_BOOK *bk) {
  dbReload(AE_Book_TableType_None);
}



void FM_CountersCounterTab::dbClosing() {
  m_counterTree->clearItems();
}



void FM_CountersCounterTab::dbClosed() {
}



void FM_CountersCounterTab::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_CounterTypeGroup ||
      tt==AE_Book_TableType_CounterType ||
      tt==AE_Book_TableType_None) {
    FF_App *app;
    AE_BOOK *b;
    AE_GROUP *gr;
    AE_COUNTER_TYPE *ct;
    AQDB_ID currentCounterId=0;
    AQDB_ID currentGrId=0;
  
    gr=m_counterTree->getCurrentGroup();
    if (gr)
      currentGrId=AE_Group_GetId(gr);
    ct=m_counterTree->getCurrentObject();
    if (ct)
      currentCounterId=AE_CounterType_GetId(ct);
  
    m_counterTree->clearItems();
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AE_GROUP_TREE *gt;
      int rv;
      AE_COUNTER_TYPE_LIST *bal;
      int i;

      gt=AE_Group_Tree_new();
      rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_CounterTypeGroup, gt);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	m_counterTree->addGroups(gt);
      }
      AE_Group_Tree_free(gt);

      /* read counters */
      bal=AE_CounterType_List_new();
      rv=AE_Book_ReadCounterTypeList(b, AE_Book_TableType_CounterType, bal);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	AE_COUNTER_TYPE *ct;
  
	while ( (ct=AE_CounterType_List_First(bal)) ) {
	  AE_CounterType_List_Del(ct);

          /* add counter */
          m_counterTree->addObject(ct, false, 0);
        }
      }
      AE_CounterType_List_free(bal);

      for (i=0; i<m_counterTree->getNumHeaders(); i++) {
	FXbool b;
    
	b=m_counterTree->getHeaderArrowDir(i);
	if (b!=MAYBE) {
	  if (b==TRUE)
	    m_counterTree->sortByColumn(i, true);
	  else
	    m_counterTree->sortByColumn(i, false);
	  break;
	}
      }

      m_counterTree->expandItems();
    }

    if (currentCounterId!=0) {
      FF_CounterTypeTreeItem *item=m_counterTree->findObjectItem(currentCounterId);
      if (item) {
	item->setSelected(true);
	m_counterTree->makeItemVisible(item);
	m_counterTree->setCurrentItem(item);
      }
      else {
	DBG_ERROR(0, "Item not found");
      }
    }
    else if (currentGrId!=0) {
      FF_GroupTreeItem *item=m_counterTree->findGroupItem(currentGrId);
      if (item) {
	item->setSelected(true);
	m_counterTree->makeItemVisible(item);
	m_counterTree->setCurrentItem(item);
      }
      else {
	DBG_ERROR(0, "Item not found");
      }
    }
  }
}



void FM_CountersCounterTab::reset() {
}






