/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_COUNTERSCTAB_HPP
#define FM_COUNTERSCTAB_HPP


#include "ff_countertypetree.hpp"
#include "ff_modtabpage.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_CounterTypeTree;
class FM_CountersPage;


class FM_CountersCounterTab: public FF_ModTabPage {
  FXDECLARE(FM_CountersCounterTab)

public:
  enum {
    ID_COUNTER_TREE=FXPacker::ID_LAST,

    ID_BUTTON_ADDGROUP,
    ID_BUTTON_EDITGROUP,
    ID_BUTTON_DELGROUP,

    ID_BUTTON_ADDCOUNTER,
    ID_BUTTON_EDITCOUNTER,
    ID_BUTTON_DELCOUNTER,

    ID_LAST
  };

public:
  FM_CountersCounterTab(const FXString &pageId,
                        FM_CountersPage *mainPage,
                        FXuint opts=0,
                        FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_CountersCounterTab();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void reset();

  void create();

  void openCounterPage(const AE_COUNTER_TYPE *ct);

  long onCmdAddGroup(FXObject*, FXSelector, void*);
  long onCmdEditGroup(FXObject*, FXSelector, void*);
  long onCmdDelGroup(FXObject*, FXSelector, void*);

  long onCmdAddCounter(FXObject*, FXSelector, void*);
  long onCmdEditCounter(FXObject*, FXSelector, void*);
  long onCmdDelCounter(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeCommand(FXObject*, FXSelector, void* ptr);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  void dump(int indent);

  bool isEditing() const { return m_isEditing;};

protected:
  FM_CountersCounterTab();

  FM_CountersPage *m_mainPage;

  FF_CounterTypeTree *m_counterTree;

  bool m_isEditing;
};




#endif

