/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_counterspage.hpp"
#include "ff_app.hpp"
#include "fm_countersvtab.hpp"
#include "fm_countersctab.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FM_CountersPage::FM_CountersPage(FXComposite *p,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h)
:FF_ModTabBook(p, opts, x, y, w, h)
,m_counterTab(NULL)
{
  /* create tab 1 */
  addTabHeader(I18N("Counters"), false);
  m_counterTab=new FM_CountersCounterTab("countersPage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(m_counterTab);
}



FM_CountersPage::FM_CountersPage()
:FF_ModTabBook()
,m_counterTab(NULL)
{
}



FM_CountersPage::~FM_CountersPage() {
}



void FM_CountersPage::openCounterPage(const AE_COUNTER_TYPE *ct) {
  char nbuf[64];
  FXString pageId;
  FF_ModTabPage *p;
  AQDB_ID id;

  id=AE_CounterType_GetId(ct);
  /* TODO: prepend book id */
  snprintf(nbuf, sizeof(nbuf)-1, "%lld", (unsigned long long int) id);
  nbuf[sizeof(nbuf)-1]=0;

  pageId=FXString(nbuf);
  p=findPageById(pageId);
  if (p==NULL) {
    const char *s;
    FXString title;
    FXTabItem *tab;

    /* create page */
    s=AE_CounterType_GetName(ct);
    if (s==NULL || *s==0)
      s=I18N("FM_CountersPage|unknown");
    title=FXString(s);

    DBG_DEBUG(0, "Creating page %s (%s)", title.text(), pageId.text());
    tab=addTabHeader(title, true);
    p=new FM_CountersValueTab(ct, pageId, this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
    tab->create();
    p->create();
    addTabPage(p);
    p->init(m_config);
  }
  else {
    DBG_DEBUG(0, "Page %s already open, changing to it", pageId.text());
  }

  if (p)
    setCurrentPage(p);
}



