/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FM_COUNTERSVTAB_HPP
#define FM_COUNTERSVTAB_HPP


#include "ff_countervaluetree.hpp"
#include "ff_modtabpage.hpp"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/db.h>

#include <fx.h>

#include <list>


class FF_CounterValueTree;
class FM_CountersPage;


class FM_CountersValueTab: public FF_ModTabPage {
  FXDECLARE(FM_CountersValueTab)

public:
  enum {
    ID_VALUE_TREE=FXPacker::ID_LAST,

    ID_BUTTON_ADDVALUE,
    ID_BUTTON_EDITVALUE,
    ID_BUTTON_DELVALUE,

    ID_LAST
  };

public:
  FM_CountersValueTab(const AE_COUNTER_TYPE *ct,
                      const FXString &pageId,
                      FM_CountersPage *mainPage,
                      FXuint opts=0,
                      FXint x=0, FXint y=0, FXint w=0, FXint h=0);
  ~FM_CountersValueTab();

  void init(GWEN_DB_NODE *db);
  void fini(GWEN_DB_NODE *db);

  void dbOpened(AE_BOOK *bk);
  void dbClosing();
  void dbClosed();
  void dbReload(AE_BOOK_TABLE_TYPE tt);

  void reset();

  void create();

  long onCmdAddValue(FXObject*, FXSelector, void*);
  long onCmdEditValue(FXObject*, FXSelector, void*);
  long onCmdDelValue(FXObject*, FXSelector, void*);

  long onTreeChanged(FXObject*, FXSelector, void*);
  long onTreeCommand(FXObject*, FXSelector, void* ptr);
  long onTreeRightBtnPress(FXObject*, FXSelector, void* ptr);

  bool isEditing() const { return m_isEditing;};

protected:
  FM_CountersValueTab();
  void reload();

  FM_CountersPage *m_mainPage;
  AE_COUNTER_TYPE *m_counterType;

  FF_CounterValueTree *m_valueTree;
  int m_numValues;

  bool m_isEditing;
};




#endif

