/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_app.hpp"
#include "fm_outboxpage.hpp"
#include "ff_bankingjobtree.hpp"

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_gui.hpp>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#include <stdlib.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_OutboxPage) FM_OutboxPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_OutboxPage::ID_BUTTON_SENDALL,
            FM_OutboxPage::onCmdSendAll),
  FXMAPFUNC(SEL_COMMAND, FM_OutboxPage::ID_BUTTON_SENDSEL,
            FM_OutboxPage::onCmdSendSelected),
  FXMAPFUNC(SEL_COMMAND, FM_OutboxPage::ID_BUTTON_ABORTSEL,
            FM_OutboxPage::onCmdAbortSelected),
};


            
FXIMPLEMENT(FM_OutboxPage, FXHorizontalFrame,
            FM_OutboxPageMap, ARRAYNUMBER(FM_OutboxPageMap))





FM_OutboxPage::FM_OutboxPage(FM_Outbox *mod,
                             FXComposite *p,
                             FXuint opts,
                             FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_module(mod)
,m_jobList(NULL)
,m_sendAllButton(NULL)
,m_sendSelectedButton(NULL)
,m_sessionNumber(1)
{
  FXVerticalFrame *vf;
  FF_App *app;
  FXIcon *ic;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_jobList=new FF_BankingJobTree(this, this, ID_JOBLIST,
                                  LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                  FOLDINGLIST_SHOWS_LINES |
                                  FOLDINGLIST_SHOWS_BOXES |
                                  FOLDINGLIST_ROOT_BOXES |
                                  FOLDINGLIST_EXTENDEDSELECT |
                                  FRAME_SUNKEN|FRAME_THICK);

  ic=app->getIcon("minifolderopen");
  if (ic)
    m_jobList->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_jobList->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  m_sendAllButton=new THEMEBUTTON(vf,
				  I18N("Send All"),
				  NULL,
				  this, ID_BUTTON_SENDALL,
				  BUTTON_NORMAL);

  m_sendSelectedButton=new THEMEBUTTON(vf,
				       I18N("Send Selected"),
				       NULL,
				       this, ID_BUTTON_SENDSEL,
				       BUTTON_NORMAL);

  m_abortSelectedButton=new THEMEBUTTON(vf,
                                        I18N("Abort Selected"),
                                        NULL,
                                        this, ID_BUTTON_ABORTSEL,
                                        BUTTON_NORMAL);
}



FM_OutboxPage::FM_OutboxPage()
:FXHorizontalFrame()
,m_jobList(NULL)
,m_sendAllButton(NULL)
,m_sendSelectedButton(NULL)
,m_sessionNumber(0)
{
}



FM_OutboxPage::~FM_OutboxPage() {
}



void FM_OutboxPage::create() {
  FXHorizontalFrame::create();
  m_jobList->create();
  m_sendAllButton->create();
  m_sendSelectedButton->create();
}



void FM_OutboxPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "jobPage");
  if (dbT) {
    int i;

    for (i=0; i<m_jobList->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_jobList->getHeaderSize(i));
      m_jobList->setHeaderSize(i, j);
    }
  }
}



void FM_OutboxPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "jobPage");
  for (i=0; i<m_jobList->getNumHeaders(); i++) {
    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_jobList->getHeaderSize(i));
  }
}



void FM_OutboxPage::dbOpened(AE_BOOK *bk) {
}



void FM_OutboxPage::dbClosing() {
  FF_App *app;
  AB_TRANSACTION_LIST *jl;

  app=dynamic_cast<FF_App*>(getApp());

  jl=app->getJobList();
  if (jl) {
    AB_TRANSACTION *j;

    j=AB_Transaction_List_First(jl);
    while(j) {
      /* cancel jobs */
      switch(AB_Transaction_GetStatus(j)) {
      case AB_Transaction_StatusAccepted:
      case AB_Transaction_StatusRejected:
      case AB_Transaction_StatusPending:
      case AB_Transaction_StatusAutoReconciled:
      case AB_Transaction_StatusManuallyReconciled:
      case AB_Transaction_StatusRevoked:
      case AB_Transaction_StatusAborted:
      case AB_Transaction_StatusError:
        break;
      case AB_Transaction_StatusEnqueued:
      case AB_Transaction_StatusUnknown:
      case AB_Transaction_StatusNone:
      case AB_Transaction_StatusSending:
      case AB_Transaction_StatusSent:
        DBG_ERROR(0, "Cancelling job");
        app->cancelJob(j, 1);
      }

      j=AB_Transaction_List_Next(j);
    } /* while (j) */
    AB_Transaction_List_Clear(jl);
    m_jobList->clearItems();
  }
}



void FM_OutboxPage::dbClosed() {
}



void FM_OutboxPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  DBG_ERROR(0, "Reloaded type %d", (int) tt);
}



void FM_OutboxPage::enter() {
}



void FM_OutboxPage::leave() {
}



void FM_OutboxPage::jobListUpdated() {
  FF_App *app=dynamic_cast<FF_App*>(getApp());
  AB_TRANSACTION *j=m_jobList->getCurrentBankingJob();
  AE_GROUP *gr=m_jobList->getCurrentGroup();
  AQDB_ID jId=0;
  AQDB_ID grId=0;
  AE_BOOK *b;

  if (j)
    jId=AB_Transaction_GetUniqueId(j);
  if (gr)
    grId=AE_Group_GetId(gr);

  m_jobList->clearItems();

  b=app->getBook();
  if (b) {
    AB_TRANSACTION_LIST *jl;

    jl=app->getJobList();
    if (jl) {
      AB_TRANSACTION *j;

      j=AB_Transaction_List_First(jl);
      while(j) {
        FF_GroupTreeItem *grItem;
        FF_BankingJobTreeItem *jItem;

          {
            int sessionId;

            sessionId=AB_Transaction_GetSessionId(j);
            if (sessionId==0) {
              sessionId=m_sessionNumber;
              AB_Transaction_SetSessionId(j, sessionId);
            }

            grItem=m_jobList->findGroupItem(sessionId);
            if (grItem==NULL) {
              gr=AE_Group_new();
              AE_Group_SetId(gr, sessionId);
              AE_Group_SetName(gr, FXStringFormat("Session-%02d", sessionId).text());

              grItem=new FF_GroupTreeItem(gr,
                                          m_jobList->getIconOpen(),
                                          m_jobList->getIconClosed());
              m_jobList->appendItem(NULL, grItem, false);
            }
          }


        jItem=new FF_BankingJobTreeItem(j, NULL, NULL);
        m_jobList->appendItem(grItem, jItem, false);

        j=AB_Transaction_List_Next(j);
      }
    }

    m_jobList->sortItems();
    m_jobList->expandItems();

    if (jId!=0) {
      FF_BankingJobTreeItem *item=m_jobList->findBankingJobItem(jId);
      if (item) {
        item->setSelected(true);
        m_jobList->makeItemVisible(item);
        m_jobList->setCurrentItem(item);
      }
      else {
        DBG_DEBUG(0, "Item not found");
      }
    }
    else if (grId!=0) {
      FF_GroupTreeItem *item=m_jobList->findGroupItem(grId);
      if (item) {
        item->setSelected(true);
        m_jobList->makeItemVisible(item);
        m_jobList->setCurrentItem(item);
      }
      else {
        DBG_ERROR(0, "Item not found");
      }
    }
  }
}



long FM_OutboxPage::onCmdSendAll(FXObject*, FXSelector, void* ptr) {
  FF_App *app;
  AB_TRANSACTION_LIST2 *jlCopy;
  AB_TRANSACTION_LIST *jlApp;
  AB_TRANSACTION *j;
  int rv;

  app=dynamic_cast<FF_App*>(getApp());

  jlApp=app->getJobList();

  /* copy jobs without status (i.e. new) */
  jlCopy=AB_Transaction_List2_new();
  j=AB_Transaction_List_First(jlApp);
  while(j) {
    if (AB_Transaction_GetStatus(j)==AB_Transaction_StatusNone) {
      AB_Transaction_SetStatus(j, AB_Transaction_StatusEnqueued);
      AB_Transaction_List2_PushBack(jlCopy, j);
    }
    j=AB_Transaction_List_Next(j);
  } /* while (j) */

  /* we changed the application queue, tell the other modules */
  app->signalJobListUpdated();

  if (AB_Transaction_List2_GetSize(jlCopy)==0) {
    DBG_ERROR(0, "Nothing to send");
    AB_Transaction_List2_free(jlCopy);
    return 1;
  }

  /* send jobs */
  rv=sendJobList(jlCopy);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }

  /* don't call @ref AB_Job_List2_FreeAll() here because the jobs
   * are still also in the application queue */
  AB_Transaction_List2_free(jlCopy);

  return 1;
}



long FM_OutboxPage::onCmdSendSelected(FXObject*, FXSelector, void* ptr) {
  FF_App *app;
  AB_TRANSACTION_LIST2 *jl;
  AB_TRANSACTION_LIST2 *jlCopy;
  AB_TRANSACTION_LIST2_ITERATOR *jit;
  int rv;

  app=dynamic_cast<FF_App*>(getApp());

  /* copy queued jobs */
  jl=AB_Transaction_List2_new();
  m_jobList->getSelectedBankingJobs(jl);

  jlCopy=AB_Transaction_List2_new();
  jit=AB_Transaction_List2_First(jl);
  if (jit) {
    AB_TRANSACTION *j;

    j=AB_Transaction_List2Iterator_Data(jit);
    while(j) {
      if (AB_Transaction_GetStatus(j)==AB_Transaction_StatusNone) {
        AB_Transaction_List2_PushBack(jlCopy, j);
        AB_Transaction_SetStatus(j, AB_Transaction_StatusEnqueued);
      }
      j=AB_Transaction_List2Iterator_Next(jit);
    }
    AB_Transaction_List2Iterator_free(jit);
  }

  /* we changed the application queue, tell the other modules */
  app->signalJobListUpdated();

  if (AB_Transaction_List2_GetSize(jlCopy)==0) {
    DBG_ERROR(0, "Nothing to send");
    AB_Transaction_List2_free(jlCopy);
    AB_Transaction_List2_free(jl);
    return 1;
  }

  /* send jobs */
  rv=sendJobList(jlCopy);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }

  /* don't call @ref AB_Job_List2_FreeAll() here because the jobs
   * are still also in the application queue */
  AB_Transaction_List2_free(jlCopy);
  AB_Transaction_List2_free(jl);

  return 1;
}



long FM_OutboxPage::onCmdAbortSelected(FXObject*, FXSelector, void* ptr) {
  FF_App *app;
  AB_TRANSACTION_LIST2 *jl;
  AB_TRANSACTION_LIST2_ITERATOR *jit;
  bool someAborted=false;

  app=dynamic_cast<FF_App*>(getApp());
  DBG_ERROR(0, "Aborting selected jobs");

  /* copy queued jobs */
  jl=AB_Transaction_List2_new();
  m_jobList->getSelectedBankingJobs(jl);
  DBG_ERROR(0, "Found %d jobs", AB_Transaction_List2_GetSize(jl));

  jit=AB_Transaction_List2_First(jl);
  if (jit) {
    AB_TRANSACTION *j;

    j=AB_Transaction_List2Iterator_Data(jit);
    while(j) {
      switch(AB_Transaction_GetStatus(j)) {
      case AB_Transaction_StatusAccepted:
      case AB_Transaction_StatusRejected:
      case AB_Transaction_StatusPending:
      case AB_Transaction_StatusAutoReconciled:
      case AB_Transaction_StatusManuallyReconciled:
      case AB_Transaction_StatusRevoked:
      case AB_Transaction_StatusAborted:
      case AB_Transaction_StatusError:
        break;
      case AB_Transaction_StatusEnqueued:
      case AB_Transaction_StatusUnknown:
      case AB_Transaction_StatusNone:
      case AB_Transaction_StatusSending:
      case AB_Transaction_StatusSent:
        DBG_ERROR(0, "Cancelling job");
        app->cancelJob(j, 1);
        someAborted=true;
      }
      j=AB_Transaction_List2Iterator_Next(jit);
    }
    AB_Transaction_List2Iterator_free(jit);
  }
  AB_Transaction_List2_free(jl);

  if (someAborted)
    m_sessionNumber++;

  /* we changed the application queue, tell the other modules */
  app->signalJobListUpdated();

  return 1;
}



int FM_OutboxPage::sendJobList(AB_TRANSACTION_LIST2 *jobs) {
  FF_App *app;
  AB_IMEXPORTER_CONTEXT *ctx;
  int rv;
  AB_TRANSACTION_LIST2_ITERATOR *jit;

  app=dynamic_cast<FF_App*>(getApp());

  /* set session number in jobs (if there is none already) */
  jit=AB_Transaction_List2_First(jobs);
  if (jit) {
    AB_TRANSACTION *j;

    j=AB_Transaction_List2Iterator_Data(jit);
    while(j) {
      if (AB_Transaction_GetSessionId(j)==0)
        AB_Transaction_SetSessionId(j, m_sessionNumber);
      j=AB_Transaction_List2Iterator_Next(jit);
    }
    AB_Transaction_List2Iterator_free(jit);
  }

  /* clear password DB */
  GWEN_Gui_SetPasswordStatus(NULL, NULL, GWEN_Gui_PasswordStatus_Remove, 0);

  ctx=AB_ImExporterContext_new();

  /* inform other modules */
  app->signalAboutToSendJobs();

  /* send jobs */
  rv=AB_Banking_SendCommands(app->getBanking(), jobs, ctx);

  /* increase session number in any case */
  m_sessionNumber++;

  /* the status of the jobs has changed, notify other modules */
  app->signalJobListUpdated();

  /* clear password cache in any case */
  GWEN_Gui_SetPasswordStatus(NULL, NULL, GWEN_Gui_PasswordStatus_Remove, 0);
  if (rv==GWEN_ERROR_USER_ABORTED) {
    AB_ImExporterContext_free(ctx);
    return rv;
  }

  if (1) {
    char idbuf[256];
    GWEN_DB_NODE *dbD;

    dbD=GWEN_DB_Group_new("context");
    AB_ImExporterContext_toDb(ctx, dbD);

    rv=GWEN_ConfigMgr_GetUniqueId(app->getConfigManager(), "bankingctx", idbuf, sizeof(idbuf)-1);
    if (rv<0) {
      DBG_ERROR(0, "Error getting unique id for context (%d)", rv);
    }
    else {
      rv=GWEN_ConfigMgr_LockGroup(app->getConfigManager(), "bankingctx", idbuf);
      if (rv<0) {
        DBG_ERROR(0, "Error locking db for context (%d)", rv);
      }
      else {
        rv=GWEN_ConfigMgr_SetGroup(app->getConfigManager(), "bankingctx", idbuf, dbD);
        GWEN_ConfigMgr_UnlockGroup(app->getConfigManager(), "bankingctx", idbuf);
        if (rv<0) {
          DBG_ERROR(0, "Error storing context (%d)", rv);
        }
      }
    }
    GWEN_DB_Group_free(dbD);
  }

  if (getenv("AQF_SAVE_CTX")) {
    GWEN_DB_NODE *dbD;

    dbD=GWEN_DB_Group_new("context");
    AB_ImExporterContext_toDb(ctx, dbD);
    GWEN_DB_WriteFile(dbD, "/tmp/test.ctx", GWEN_DB_FLAGS_DEFAULT);
    GWEN_DB_Group_free(dbD);
  }

  rv=app->importContext(this, ctx);
  AB_ImExporterContext_free(ctx);

  return rv;
}





