/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_reportspage.hpp"
#include "fm_greportstab.hpp"
#include "fm_greporttab.hpp"
#include "ff_app.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FM_ReportsPage::FM_ReportsPage(FXComposite *p,
                               FXuint opts,
                               FXint x, FXint y, FXint w, FXint h)
:FF_ModTabBook(p, opts, x, y, w, h)
,m_graphReportsTab(NULL)
,m_categoryGraphReportsTab(NULL)
,m_counterGraphReportsTab(NULL)
{
  /* create tab 1 */
  addTabHeader(I18N("Accounts"), false);
  m_graphReportsTab=new FM_GraphicsReportsTab(AE_Book_TableType_GraphicReportGroup,
                                              AE_Book_TableType_GraphicReportSettings,
                                              "graphReportsPage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(m_graphReportsTab);

  /* create tab 2 */
  addTabHeader(I18N("Categories"), false);
  m_categoryGraphReportsTab=new FM_GraphicsReportsTab(AE_Book_TableType_CategoryGraphicReportGroup,
                                                      AE_Book_TableType_CategoryGraphicReportSettings,
                                                      "categoryGraphReportsPage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(m_categoryGraphReportsTab);

  /* create tab 3 */
  addTabHeader(I18N("Counters"), false);
  m_counterGraphReportsTab=new FM_GraphicsReportsTab(AE_Book_TableType_CounterGraphicReportGroup,
                                                     AE_Book_TableType_CounterGraphicReportSettings,
                                                     "counterGraphReportsPage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(m_counterGraphReportsTab);
}



FM_ReportsPage::FM_ReportsPage()
:FF_ModTabBook()
,m_graphReportsTab(NULL)
,m_categoryGraphReportsTab(NULL)
,m_counterGraphReportsTab(NULL)
{
}



FM_ReportsPage::~FM_ReportsPage() {
}



void FM_ReportsPage::openGraphicsReportPage(const AF_REPORT2_SETTINGS *settings) {
  FXTabItem *th;
  FM_GraphicsReportTab *tab;
  const char *s;

  s=AF_Report2_Settings_GetName(settings);
  DBG_ERROR(AE_LOGDOMAIN, "Report name: %s", s);
  if (s && *s)
    th=addTabHeader(s, true);
  else
    th=addTabHeader(I18N("Report"), true);

  tab=new FM_GraphicsReportTab(AE_Book_TableType_GraphicReportSettings,
                               settings, "graphReportPage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(tab);

  th->create();
  tab->create();

  setCurrentPage(tab);
}



void FM_ReportsPage::openCounterGraphicsReportPage(const AF_REPORT2_SETTINGS *settings) {
  FXTabItem *th;
  FM_GraphicsReportTab *tab;
  const char *s;

  s=AF_Report2_Settings_GetName(settings);
  DBG_ERROR(AE_LOGDOMAIN, "Report name: %s", s);
  if (s && *s)
    th=addTabHeader(s, true);
  else
    th=addTabHeader(I18N("Report"), true);

  tab=new FM_GraphicsReportTab(AE_Book_TableType_CounterGraphicReportSettings,
                               settings, "graphReportPage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(tab);

  th->create();
  tab->create();

  setCurrentPage(tab);
}



void FM_ReportsPage::openCategoryGraphicsReportPage(const AF_REPORT2_SETTINGS *settings) {
  FXTabItem *th;
  FM_GraphicsReportTab *tab;
  const char *s;

  s=AF_Report2_Settings_GetName(settings);
  DBG_ERROR(AE_LOGDOMAIN, "Report name: %s", s);
  if (s && *s)
    th=addTabHeader(s, true);
  else
    th=addTabHeader(I18N("Report"), true);

  tab=new FM_GraphicsReportTab(AE_Book_TableType_CategoryGraphicReportSettings,
                               settings, "graphReportPage", this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  addTabPage(tab);

  th->create();
  tab->create();

  setCurrentPage(tab);
}





