/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_COUNTERTYPETREE_HPP
#define FF_COUNTERTYPETREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_counter_type.h>



class FF_CounterTypeTreeItem: public FF_GroupTreeItem {
public:
  FF_CounterTypeTreeItem(AE_COUNTER_TYPE *st, void* ptr=NULL);

  virtual ~FF_CounterTypeTreeItem();

  AE_COUNTER_TYPE *getObject() const { return m_object;};

  virtual FXString getItemText(const FXXCheckTree* list) const;

protected:
  AE_COUNTER_TYPE *m_object;
};



class FF_CounterTypeTree: public FF_GroupTree {
  friend class FF_CounterTypeTreeItem;
public:
  enum {
    USECOLOR_NORMAL=0,
    USECOLOR_NOTICE,
    USECOLOR_WARNING
  };

  enum {
    COLUMN_NONE=0,
    COLUMN_NAME=1,
    COLUMN_DESCRIPTION,
    COLUMN_XLABEL,
    COLUMN_YLABEL,
    COLUMN_ZLABEL,

    COLUMN_MAX=32
  };

  FF_CounterTypeTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                     FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_CounterTypeTree();

  void addColumn(int c, int w=100);
  void addColumn(const char *colName, int c, int w);

  FF_CounterTypeTreeItem *findObjectItem(AQDB_ID id);
  FF_CounterTypeTreeItem *findObjectItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_CounterTypeTreeItem *findObjectItem(const AE_COUNTER_TYPE *qba);

  void addObject(AE_COUNTER_TYPE *st, bool direct, int useColor);
  void addObjects(AE_COUNTER_TYPE_LIST2 *sl, int copyMode, bool direct);

  AE_COUNTER_TYPE *getCurrentObject();
  AE_COUNTER_TYPE *getFirstSelectedObject();
  void getSelectedObjects(AE_COUNTER_TYPE_LIST2 *sl, int copyMode);

  void selectObject(AQDB_ID id, bool b);
  void selectObject(const AE_COUNTER_TYPE *st, bool b);
  void makeObjectVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);
  void addColumn(int c);

  void reload();

protected:
  FF_CounterTypeTree();

  static FXint sortFunctionUpName(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownName(const FXFoldingItem *it1, const FXFoldingItem *it2);

  int m_columns;
  int m_columnTypes[COLUMN_MAX];
};



#endif




