/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef FF_COUNTERTYPETREE_HPP
#define FF_COUNTERTYPETREE_HPP

#include "ff_grouptree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_counter_value.h>



class FF_CounterValueTreeItem: public FF_GroupTreeItem {
public:
  FF_CounterValueTreeItem(AE_COUNTER_VALUE *st, void* ptr=NULL);

  virtual ~FF_CounterValueTreeItem();

  AE_COUNTER_VALUE *getObject() const { return m_object;};

  virtual FXString getItemText(const FXXCheckTree* list) const;

protected:
  static FXString _valueToString(const AQDB_VALUE *v);

  AE_COUNTER_VALUE *m_object;
};



class FF_CounterValueTree: public FF_GroupTree {
  friend class FF_CounterValueTreeItem;
public:
  enum {
    USECOLOR_NORMAL=0,
    USECOLOR_NOTICE,
    USECOLOR_WARNING
  };

  enum {
    COLUMN_NONE=0,
    COLUMN_DATE=1,
    COLUMN_VALUEX,
    COLUMN_VALUEY,
    COLUMN_VALUEZ,
    COLUMN_NOTE,

    COLUMN_DIFFDAYS,
    COLUMN_DIFFX,
    COLUMN_DIFFY,
    COLUMN_DIFFZ,

    COLUMN_XPERDAY,
    COLUMN_XPERMONTH,
    COLUMN_XPERYEAR,

    COLUMN_YPERDAY,
    COLUMN_YPERMONTH,
    COLUMN_YPERYEAR,

    COLUMN_ZPERDAY,
    COLUMN_ZPERMONTH,
    COLUMN_ZPERYEAR,

    COLUMN_TOTALXPERDAY,
    COLUMN_TOTALXPERMONTH,
    COLUMN_TOTALXPERYEAR,

    COLUMN_TOTALYPERDAY,
    COLUMN_TOTALYPERMONTH,
    COLUMN_TOTALYPERYEAR,

    COLUMN_TOTALZPERDAY,
    COLUMN_TOTALZPERMONTH,
    COLUMN_TOTALZPERYEAR,

    COLUMN_MAX=32
  };

  FF_CounterValueTree(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
                      FXint x=0, FXint y=0, FXint w=0, FXint h=0);

  virtual ~FF_CounterValueTree();

  void addColumn(int c, int w=100);
  void addColumn(const char *colName, int c, int w);

  FF_CounterValueTreeItem *findObjectItem(AQDB_ID id);
  FF_CounterValueTreeItem *findObjectItem(FF_GroupTreeItem *grItem, AQDB_ID id);
  FF_CounterValueTreeItem *findObjectItem(const AE_COUNTER_VALUE *qba);

  void addObject(AE_COUNTER_VALUE *st, int useColor);
  void addObjects(AE_COUNTER_VALUE_LIST2 *sl, int copyMode);

  AE_COUNTER_VALUE *getCurrentObject();
  AE_COUNTER_VALUE *getFirstSelectedObject();
  void getSelectedObjects(AE_COUNTER_VALUE_LIST2 *sl, int copyMode);

  void selectObject(AQDB_ID id, bool b);
  void selectObject(const AE_COUNTER_VALUE *st, bool b);
  void makeObjectVisible(AQDB_ID id);

  void create();

  void sortByColumn(int c, bool up);
  void addColumn(int c);

  void reload();

protected:
  FF_CounterValueTree();


  static FXint sortFunctionUpDate(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownDate(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpValueX(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownValueX(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpValueY(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownValueY(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpValueZ(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownValueZ(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpDiffDays(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownDiffDays(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpDiffX(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownDiffX(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpDiffY(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownDiffY(const FXFoldingItem *it1, const FXFoldingItem *it2);

  static FXint sortFunctionUpDiffZ(const FXFoldingItem *it1, const FXFoldingItem *it2);
  static FXint sortFunctionDownDiffZ(const FXFoldingItem *it1, const FXFoldingItem *it2);


  int m_columns;
  int m_columnTypes[COLUMN_MAX];
};



#endif




