/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#include <aqfinance/engine/book/ae_utils.h>



void AE_CounterValues_CalculateDiffValues(AE_COUNTER_VALUE *cv, const AE_COUNTER_VALUE *lastValue)
{
  if (lastValue) {
    const GWEN_DATE *d1;
    const GWEN_DATE *d2;
    const AQDB_VALUE *v1;
    const AQDB_VALUE *v2;
    int dDiff=0;

    /* calculate diff date */
    d1=AE_CounterValue_GetDate(lastValue);
    d2=AE_CounterValue_GetDate(cv);
    if (d1 && d2) {
      dDiff=GWEN_Date_Diff(d2, d1);
      AE_CounterValue_SetDiffDays(cv, dDiff);
    }

    if (dDiff) {
      /* calculate diffX */
      v1=AE_CounterValue_GetValueX(lastValue);
      v2=AE_CounterValue_GetValueX(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;

        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        AE_CounterValue_SetDiffX(cv, vDiff);
        AQDB_Value_free(vDiff);
      }

      /* calculate diffY */
      v1=AE_CounterValue_GetValueY(lastValue);
      v2=AE_CounterValue_GetValueY(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;

        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        AE_CounterValue_SetDiffY(cv, vDiff);
        AQDB_Value_free(vDiff);
      }

      /* calculate diffZ */
      v1=AE_CounterValue_GetValueZ(lastValue);
      v2=AE_CounterValue_GetValueZ(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;

        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        AE_CounterValue_SetDiffZ(cv, vDiff);
        AQDB_Value_free(vDiff);
      }

      /* calculate XPerDay */
      v1=AE_CounterValue_GetValueX(lastValue);
      v2=AE_CounterValue_GetValueX(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;
        AQDB_VALUE *vMult;

        /* calculate x per day */
        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        vMult=AQDB_Value_fromDouble(dDiff);
        AQDB_Value_DivValue(vDiff, vMult);
        AE_CounterValue_SetXPerDay(cv, vDiff);

        /* calculate x per month */
        AQDB_Value_SetValueFromDouble(vMult, 30);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetXPerMonth(cv, vMult);


        /* calculate x per year */
        AQDB_Value_SetValueFromDouble(vMult, 365);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetXPerYear(cv, vMult);

        AQDB_Value_free(vMult);
        AQDB_Value_free(vDiff);
      }

      /* calculate YPerDay */
      v1=AE_CounterValue_GetValueY(lastValue);
      v2=AE_CounterValue_GetValueY(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;
        AQDB_VALUE *vMult;

        /* calculate x per day */
        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        vMult=AQDB_Value_fromDouble(dDiff);
        AQDB_Value_DivValue(vDiff, vMult);
        AE_CounterValue_SetYPerDay(cv, vDiff);

        /* calculate x per month */
        AQDB_Value_SetValueFromDouble(vMult, 30);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetYPerMonth(cv, vMult);


        /* calculate x per year */
        AQDB_Value_SetValueFromDouble(vMult, 365);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetYPerYear(cv, vMult);

        AQDB_Value_free(vMult);
        AQDB_Value_free(vDiff);
      }

      /* calculate ZPerDay */
      v1=AE_CounterValue_GetValueZ(lastValue);
      v2=AE_CounterValue_GetValueZ(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;
        AQDB_VALUE *vMult;

        /* calculate x per day */
        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        vMult=AQDB_Value_fromDouble(dDiff);
        AQDB_Value_DivValue(vDiff, vMult);
        AE_CounterValue_SetZPerDay(cv, vDiff);

        /* calculate x per month */
        AQDB_Value_SetValueFromDouble(vMult, 30);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetZPerMonth(cv, vMult);


        /* calculate x per year */
        AQDB_Value_SetValueFromDouble(vMult, 365);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetZPerYear(cv, vMult);

        AQDB_Value_free(vMult);
        AQDB_Value_free(vDiff);
      }
    }
  }
}




void AE_CounterValues_CalculateTotalValues(AE_COUNTER_VALUE *cv, const AE_COUNTER_VALUE *firstValue)
{
  if (firstValue) {
    const GWEN_DATE *d1;
    const GWEN_DATE *d2;
    int dDiff=0;

    /* calculate diff date */
    d1=AE_CounterValue_GetDate(firstValue);
    d2=AE_CounterValue_GetDate(cv);
    if (d1 && d2)
      dDiff=GWEN_Date_Diff(d2, d1);

    if (dDiff) {
      const AQDB_VALUE *v1;
      const AQDB_VALUE *v2;

      /* calculate XPerDay */
      v1=AE_CounterValue_GetValueX(firstValue);
      v2=AE_CounterValue_GetValueX(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;
        AQDB_VALUE *vMult;

        /* calculate x per day */
        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        vMult=AQDB_Value_fromDouble(dDiff);
        AQDB_Value_DivValue(vDiff, vMult);
        AE_CounterValue_SetTotalXPerDay(cv, vDiff);

        /* calculate x per month */
        AQDB_Value_SetValueFromDouble(vMult, 30);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetTotalXPerMonth(cv, vMult);


        /* calculate x per year */
        AQDB_Value_SetValueFromDouble(vMult, 365);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetTotalXPerYear(cv, vMult);

        AQDB_Value_free(vMult);
        AQDB_Value_free(vDiff);
      }

      /* calculate YPerDay */
      v1=AE_CounterValue_GetValueY(firstValue);
      v2=AE_CounterValue_GetValueY(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;
        AQDB_VALUE *vMult;

        /* calculate x per day */
        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        vMult=AQDB_Value_fromDouble(dDiff);
        AQDB_Value_DivValue(vDiff, vMult);
        AE_CounterValue_SetTotalYPerDay(cv, vDiff);

        /* calculate x per month */
        AQDB_Value_SetValueFromDouble(vMult, 30);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetTotalYPerMonth(cv, vMult);


        /* calculate x per year */
        AQDB_Value_SetValueFromDouble(vMult, 365);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetTotalYPerYear(cv, vMult);

        AQDB_Value_free(vMult);
        AQDB_Value_free(vDiff);
      }

      /* calculate ZPerDay */
      v1=AE_CounterValue_GetValueZ(firstValue);
      v2=AE_CounterValue_GetValueZ(cv);
      if (v1 && v2) {
        AQDB_VALUE *vDiff;
        AQDB_VALUE *vMult;

        /* calculate x per day */
        vDiff=AQDB_Value_dup(v2);
        AQDB_Value_SubValue(vDiff, v1);
        vMult=AQDB_Value_fromDouble(dDiff);
        AQDB_Value_DivValue(vDiff, vMult);
        AE_CounterValue_SetZPerDay(cv, vDiff);

        /* calculate x per month */
        AQDB_Value_SetValueFromDouble(vMult, 30);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetTotalZPerMonth(cv, vMult);


        /* calculate x per year */
        AQDB_Value_SetValueFromDouble(vMult, 365);
        AQDB_Value_MultValue(vMult, vDiff);
        AE_CounterValue_SetTotalZPerYear(cv, vMult);

        AQDB_Value_free(vMult);
        AQDB_Value_free(vDiff);
      }
    }
  }
}



void AE_CounterValue_List_CalculateDiffValues(AE_COUNTER_VALUE_LIST *vList)
{
  AE_COUNTER_VALUE *lastValue=NULL;
  AE_COUNTER_VALUE *v;

  v=AE_CounterValue_List_First(vList);
  while (v) {
    if (lastValue)
      AE_CounterValues_CalculateDiffValues(v, lastValue);
    lastValue=v;

    v=AE_CounterValue_List_Next(v);
  }
}



void AE_CounterValue_List_CalculateTotalValues(AE_COUNTER_VALUE_LIST *vList)
{
  AE_COUNTER_VALUE *firstValue=NULL;
  AE_COUNTER_VALUE *v;

  v=AE_CounterValue_List_First(vList);
  while (v) {
    if (firstValue)
      AE_CounterValues_CalculateTotalValues(v, firstValue);
    else
      firstValue=v;

    v=AE_CounterValue_List_Next(v);
  }
}




const AQDB_VALUE *AE_CounterValues_GetFieldValueByName(const AE_COUNTER_VALUE *cv, const char *n)
{
  if (n && *n) {
    if (strcasecmp(n, "valueX")==0)
      return cv->valueX;
    else if (strcasecmp(n, "diffX")==0)
      return cv->diffX;
    else if (strcasecmp(n, "xPerDay")==0)
      return cv->xPerDay;
    else if (strcasecmp(n, "xPerMonth")==0)
      return cv->xPerMonth;
    else if (strcasecmp(n, "xPerYear")==0)
      return cv->xPerYear;
    else if (strcasecmp(n, "totalXPerDay")==0)
      return cv->totalXPerDay;
    else if (strcasecmp(n, "totalXPerMonth")==0)
      return cv->totalXPerMonth;
    else if (strcasecmp(n, "totalXPerYear")==0)
      return cv->totalXPerYear;

    else if (strcasecmp(n, "valueY")==0)
      return cv->valueY;
    else if (strcasecmp(n, "diffY")==0)
      return cv->diffY;
    else if (strcasecmp(n, "yPerDay")==0)
      return cv->yPerDay;
    else if (strcasecmp(n, "yPerMonth")==0)
      return cv->yPerMonth;
    else if (strcasecmp(n, "yPerYear")==0)
      return cv->yPerYear;
    else if (strcasecmp(n, "totalYPerDay")==0)
      return cv->totalYPerDay;
    else if (strcasecmp(n, "totalYPerMonth")==0)
      return cv->totalYPerMonth;
    else if (strcasecmp(n, "totalYPerYear")==0)
      return cv->totalYPerYear;

    else if (strcasecmp(n, "valueZ")==0)
      return cv->valueZ;
    else if (strcasecmp(n, "diffZ")==0)
      return cv->diffZ;
    else if (strcasecmp(n, "zPerDay")==0)
      return cv->zPerDay;
    else if (strcasecmp(n, "zPerMonth")==0)
      return cv->zPerMonth;
    else if (strcasecmp(n, "zPerYear")==0)
      return cv->zPerYear;
    else if (strcasecmp(n, "totalZPerDay")==0)
      return cv->totalZPerDay;
    else if (strcasecmp(n, "totalZPerMonth")==0)
      return cv->totalZPerMonth;
    else if (strcasecmp(n, "totalZPerYear")==0)
      return cv->totalZPerYear;

    else {
      DBG_ERROR(AE_LOGDOMAIN, "Invalid field name \"%s\"", n);
      return NULL;
    }
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "Missing field name");
    return NULL;
  }
}





